@@ -2,9 +2,8 @@ from rest_framework.routers import APIRootView
 
 from nautobot.circuits import filters
 from nautobot.circuits.models import Provider, CircuitTermination, CircuitType, Circuit, ProviderNetwork
-from nautobot.core.api.views import ModelViewSet
 from nautobot.dcim.api.views import PathEndpointMixin
-from nautobot.extras.api.views import CustomFieldModelViewSet, StatusViewSetMixin
+from nautobot.extras.api.views import NautobotModelViewSet, StatusViewSetMixin
 from nautobot.utilities.utils import count_related
 from . import serializers
 
@@ -23,7 +22,7 @@ class CircuitsRootView(APIRootView):
 #
 
 
-class ProviderViewSet(CustomFieldModelViewSet):
+class ProviderViewSet(NautobotModelViewSet):
     queryset = Provider.objects.prefetch_related("tags").annotate(circuit_count=count_related(Circuit, "provider"))
     serializer_class = serializers.ProviderSerializer
     filterset_class = filters.ProviderFilterSet
@@ -34,7 +33,7 @@ class ProviderViewSet(CustomFieldModelViewSet):
 #
 
 
-class CircuitTypeViewSet(CustomFieldModelViewSet):
+class CircuitTypeViewSet(NautobotModelViewSet):
     queryset = CircuitType.objects.annotate(circuit_count=count_related(Circuit, "type"))
     serializer_class = serializers.CircuitTypeSerializer
     filterset_class = filters.CircuitTypeFilterSet
@@ -45,7 +44,7 @@ class CircuitTypeViewSet(CustomFieldModelViewSet):
 #
 
 
-class CircuitViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
+class CircuitViewSet(StatusViewSetMixin, NautobotModelViewSet):
     queryset = Circuit.objects.prefetch_related(
         "status", "type", "tenant", "provider", "termination_a", "termination_z"
     ).prefetch_related("tags")
@@ -58,7 +57,7 @@ class CircuitViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
 #
 
 
-class CircuitTerminationViewSet(PathEndpointMixin, ModelViewSet):
+class CircuitTerminationViewSet(PathEndpointMixin, NautobotModelViewSet):
     queryset = CircuitTermination.objects.prefetch_related("circuit", "site", "_path__destination", "cable")
     serializer_class = serializers.CircuitTerminationSerializer
     filterset_class = filters.CircuitTerminationFilterSet
@@ -70,7 +69,7 @@ class CircuitTerminationViewSet(PathEndpointMixin, ModelViewSet):
 #
 
 
-class ProviderNetworkViewSet(CustomFieldModelViewSet):
+class ProviderNetworkViewSet(NautobotModelViewSet):
     queryset = ProviderNetwork.objects.prefetch_related("tags")
     serializer_class = serializers.ProviderNetworkSerializer
     filterset_class = filters.ProviderNetworkFilterSet
