@@ -1,3 +1,4 @@
+import logging
 from collections import defaultdict
 
 from django.contrib.contenttypes.fields import GenericForeignKey
@@ -25,12 +26,13 @@ from nautobot.utilities.utils import to_meters
 from .devices import Device
 from .device_components import FrontPort, RearPort
 
-
 __all__ = (
     "Cable",
     "CablePath",
 )
 
+logger = logging.getLogger(__name__)
+
 
 #
 # Cables
@@ -136,12 +138,20 @@ class Cable(PrimaryModel, StatusModel):
     def STATUS_CONNECTED(cls):
         """Return a cached "connected" `Status` object for later reference."""
         if getattr(cls, "__status_connected", None) is None:
-            cls.__status_connected = Status.objects.get_for_model(Cable).get(slug="connected")
+            try:
+                cls.__status_connected = Status.objects.get_for_model(Cable).get(slug="connected")
+            except Status.DoesNotExist:
+                logger.warning("Status 'connected' not found for dcim.cable")
+                return None
+
         return cls.__status_connected
 
     def clean(self):
         super().clean()
 
+        # We import this in this method due to circular importing issues.
+        from nautobot.circuits.models import CircuitTermination
+
         # Validate that termination A exists
         if not hasattr(self, "termination_a_type"):
             raise ValidationError("Termination A type has not been specified")
@@ -227,6 +237,16 @@ class Cable(PrimaryModel, StatusModel):
         ):
             raise ValidationError("A front port cannot be connected to it corresponding rear port")
 
+        # A CircuitTermination attached to a Provider Network cannot have a Cable
+        if isinstance(self.termination_a, CircuitTermination) and self.termination_a.provider_network is not None:
+            raise ValidationError(
+                {"termination_a_id": "Circuit terminations attached to a provider network may not be cabled."}
+            )
+        if isinstance(self.termination_b, CircuitTermination) and self.termination_b.provider_network is not None:
+            raise ValidationError(
+                {"termination_b_id": "Circuit terminations attached to a provider network may not be cabled."}
+            )
+
         # Check for an existing Cable connected to either termination object
         if self.termination_a.cable not in (None, self):
             raise ValidationError(
@@ -323,6 +343,7 @@ class CablePath(BaseModel):
     )
     destination_id = models.UUIDField(blank=True, null=True)
     destination = GenericForeignKey(ct_field="destination_type", fk_field="destination_id")
+    # TODO: Profile filtering on this field if it could benefit from an index
     path = JSONPathField()
     is_active = models.BooleanField(default=False)
     is_split = models.BooleanField(default=False)
