@@ -1,9 +1,19 @@
 from django.test import TestCase
 
-from nautobot.dcim.forms import DeviceForm, InterfaceCreateForm, CableCSVForm
-from nautobot.dcim.choices import DeviceFaceChoices, InterfaceTypeChoices
+from nautobot.dcim.forms import CableCSVForm, DeviceForm, InterfaceCreateForm, InterfaceCSVForm
+from nautobot.dcim.choices import DeviceFaceChoices, InterfaceStatusChoices, InterfaceTypeChoices
 
-from nautobot.dcim.models import Device, DeviceRole, DeviceType, Manufacturer, Platform, Rack, Site, Interface
+from nautobot.dcim.models import (
+    Device,
+    DeviceRole,
+    DeviceType,
+    Interface,
+    Manufacturer,
+    Platform,
+    Rack,
+    Site,
+    VirtualChassis,
+)
 from nautobot.extras.models import SecretsGroup, Status
 from nautobot.virtualization.models import Cluster, ClusterGroup, ClusterType
 
@@ -159,11 +169,13 @@ class LabelTestCase(TestCase):
 
     def test_interface_label_count_valid(self):
         """Test that a `label` can be generated for each generated `name` from `name_pattern` on InterfaceCreateForm"""
+        status_active = Status.objects.get_for_model(Interface).get(slug=InterfaceStatusChoices.STATUS_ACTIVE)
         interface_data = {
             "device": self.device.pk,
             "name_pattern": "eth[0-9]",
             "label_pattern": "Interface[0-9]",
             "type": InterfaceTypeChoices.TYPE_100ME_FIXED,
+            "status": status_active.pk,
         }
         form = InterfaceCreateForm(interface_data)
 
@@ -242,3 +254,151 @@ class TestCableCSVForm(TestCase):
         self.assertFalse(form.is_valid())
         self.assertIn("side_a_name", form.errors)
         self.assertNotIn("termination_a_id", form.errors)
+
+
+class TestInterfaceCSVForm(TestCase):
+    @classmethod
+    def setUpTestData(cls):
+        site = Site.objects.create(name="Site 1", slug="site-1")
+        manufacturer = Manufacturer.objects.create(name="Manufacturer 1", slug="manufacturer-1")
+        device_type = DeviceType.objects.create(manufacturer=manufacturer, model="Model 1", slug="model-1")
+        device_role = DeviceRole.objects.create(name="Device Role 1", slug="device-role-1")
+
+        cls.devices = (
+            Device.objects.create(
+                name="Device 1",
+                device_type=device_type,
+                device_role=device_role,
+                site=site,
+            ),
+            Device.objects.create(
+                name="Device 2",
+                device_type=device_type,
+                device_role=device_role,
+                site=site,
+            ),
+            Device.objects.create(
+                name="Device 3",
+                device_type=device_type,
+                device_role=device_role,
+                site=site,
+            ),
+        )
+
+        virtualchassis = VirtualChassis.objects.create(
+            name="Virtual Chassis 1", master=cls.devices[0], domain="domain-1"
+        )
+        Device.objects.filter(id=cls.devices[0].id).update(virtual_chassis=virtualchassis, vc_position=1)
+        Device.objects.filter(id=cls.devices[1].id).update(virtual_chassis=virtualchassis, vc_position=2)
+
+        cls.interfaces = (
+            Interface.objects.create(
+                device=cls.devices[0],
+                name="Interface 1",
+                type=InterfaceTypeChoices.TYPE_1GE_SFP,
+            ),
+            Interface.objects.create(
+                device=cls.devices[1],
+                name="Interface 2",
+                type=InterfaceTypeChoices.TYPE_1GE_FIXED,
+            ),
+            Interface.objects.create(
+                device=cls.devices[1],
+                name="Interface 3",
+                type=InterfaceTypeChoices.TYPE_LAG,
+            ),
+            Interface.objects.create(
+                device=cls.devices[2],
+                name="Interface 4",
+                type=InterfaceTypeChoices.TYPE_LAG,
+            ),
+            Interface.objects.create(
+                device=cls.devices[2],
+                name="Interface 5",
+                type=InterfaceTypeChoices.TYPE_1GE_FIXED,
+            ),
+        )
+        cls.headers_1 = {
+            "device": None,
+            "name": None,
+            "status": None,
+            "parent_interface": None,
+            "bridge": None,
+            "type": None,
+        }
+        cls.headers_2 = {
+            "device": None,
+            "name": None,
+            "status": None,
+            "lag": None,
+            "bridge": None,
+            "type": None,
+        }
+
+    def test_interface_belonging_to_common_device_or_vc_allowed(self):
+        """Test parent, bridge, and LAG interfaces belonging to common device or VC is valid"""
+
+        data_1 = {
+            "device": self.devices[0].name,
+            "name": "interface test",
+            "status": "active",
+            "parent_interface": self.interfaces[0].name,
+            "bridge": self.interfaces[2].name,
+            "type": InterfaceTypeChoices.TYPE_VIRTUAL,
+        }
+
+        form = InterfaceCSVForm(data_1, headers=self.headers_1)
+        self.assertTrue(form.is_valid())
+        form.save()
+
+        interface = Interface.objects.get(name="interface test", device=self.devices[0])
+        self.assertEqual(interface.parent_interface, self.interfaces[0])
+        self.assertEqual(interface.bridge, self.interfaces[2])
+
+        # Assert LAG
+        data_2 = {
+            "device": self.devices[0].name,
+            "name": "interface lagged",
+            "status": "active",
+            "lag": self.interfaces[2].name,
+            "bridge": self.interfaces[1].name,
+            "type": InterfaceTypeChoices.TYPE_100ME_FIXED,
+        }
+
+        form = InterfaceCSVForm(data_2, headers=self.headers_2)
+        self.assertTrue(form.is_valid())
+        form.save()
+
+        interface = Interface.objects.get(name="interface lagged", device=self.devices[0])
+        self.assertEqual(interface.lag, self.interfaces[2])
+        self.assertEqual(interface.bridge, self.interfaces[1])
+
+    def test_interface_not_belonging_to_common_device_or_vc_not_allowed(self):
+        """Test parent, bridge, and LAG interfaces not belonging to common device or VC is invalid"""
+        data = {
+            "device": self.devices[0].name,
+            "name": "interface test",
+            "status": "active",
+            "parent_interface": self.interfaces[4].name,
+            "bridge": self.interfaces[4].name,
+            "type": InterfaceTypeChoices.TYPE_VIRTUAL,
+        }
+
+        form = InterfaceCSVForm(data, headers=self.headers_1)
+        self.assertFalse(form.is_valid())
+        self.assertTrue(form.has_error("parent_interface"))
+        self.assertTrue(form.has_error("bridge"))
+
+        # Assert LAG
+        data = {
+            "device": self.devices[0].name,
+            "name": "interface lagged",
+            "status": "active",
+            "lag": self.interfaces[3].name,
+            "bridge": self.interfaces[1].name,
+            "type": InterfaceTypeChoices.TYPE_VIRTUAL,
+        }
+
+        form = InterfaceCSVForm(data, headers=self.headers_2)
+        self.assertFalse(form.is_valid())
+        self.assertTrue(form.has_error("lag"))
