@@ -1,10 +1,10 @@
+from django.conf import settings
 from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist
 from django.utils.functional import classproperty
 from drf_spectacular.utils import extend_schema_field
-from nautobot.core.api.serializers import BaseModelSerializer
-from nautobot.extras.models.secrets import SecretsGroupAssociation
 from rest_framework import serializers
+from rest_framework.reverse import reverse
 
 from nautobot.core.api import (
     ChoiceField,
@@ -13,16 +13,19 @@ from nautobot.core.api import (
     ValidatedModelSerializer,
 )
 from nautobot.core.api.exceptions import SerializerNotFound
+from nautobot.core.api.serializers import BaseModelSerializer
 from nautobot.dcim.api.nested_serializers import (
     NestedDeviceSerializer,
     NestedDeviceRoleSerializer,
     NestedDeviceTypeSerializer,
+    NestedLocationSerializer,
     NestedPlatformSerializer,
     NestedRackSerializer,
     NestedRegionSerializer,
     NestedSiteSerializer,
 )
-from nautobot.dcim.models import Device, DeviceRole, DeviceType, Platform, Rack, Region, Site
+from nautobot.dcim.models import Device, DeviceRole, DeviceType, Location, Platform, Rack, Region, Site
+from nautobot.extras.api.fields import StatusSerializerField
 from nautobot.extras.choices import (
     CustomFieldFilterLogicChoices,
     CustomFieldTypeChoices,
@@ -38,25 +41,28 @@ from nautobot.extras.models import (
     CustomFieldChoice,
     CustomLink,
     DynamicGroup,
+    DynamicGroupMembership,
     ExportTemplate,
     GitRepository,
     GraphQLQuery,
     ImageAttachment,
     Job,
+    JobHook,
     JobLogEntry,
     JobResult,
+    Note,
     ObjectChange,
     Relationship,
     RelationshipAssociation,
     ScheduledJob,
     Secret,
     SecretsGroup,
+    SecretsGroupAssociation,
     Status,
     Tag,
     Webhook,
 )
-from nautobot.extras.api.fields import StatusSerializerField
-from nautobot.extras.utils import FeatureQuery, TaggableClassesQuery
+from nautobot.extras.utils import ChangeLoggedModelsQuery, FeatureQuery, TaggableClassesQuery
 from nautobot.tenancy.api.nested_serializers import (
     NestedTenantSerializer,
     NestedTenantGroupSerializer,
@@ -64,28 +70,34 @@ from nautobot.tenancy.api.nested_serializers import (
 from nautobot.tenancy.models import Tenant, TenantGroup
 from nautobot.users.api.nested_serializers import NestedUserSerializer
 from nautobot.utilities.api import get_serializer_for_model
+from nautobot.utilities.utils import slugify_dashes_to_underscores
 from nautobot.virtualization.api.nested_serializers import (
     NestedClusterGroupSerializer,
     NestedClusterSerializer,
 )
 from nautobot.virtualization.models import Cluster, ClusterGroup
+
 from .customfields import CustomFieldModelSerializer
 from .fields import MultipleChoiceJSONField
+from .relationships import RelationshipModelSerializerMixin
 
 # Not all of these variable(s) are not actually used anywhere in this file, but required for the
 # automagically replacing a Serializer with its corresponding NestedSerializer.
 from .nested_serializers import (  # noqa: F401
+    NestedComputedFieldSerializer,
     NestedConfigContextSchemaSerializer,
     NestedConfigContextSerializer,
     NestedCustomFieldSerializer,
     NestedCustomLinkSerializer,
     NestedDynamicGroupSerializer,
+    NestedDynamicGroupMembershipSerializer,
     NestedExportTemplateSerializer,
     NestedGitRepositorySerializer,
     NestedGraphQLQuerySerializer,
     NestedImageAttachmentSerializer,
     NestedJobSerializer,
     NestedJobResultSerializer,
+    NestedNoteSerializer,
     NestedRelationshipAssociationSerializer,
     NestedRelationshipSerializer,
     NestedScheduledJobSerializer,
@@ -95,13 +107,67 @@ from .nested_serializers import (  # noqa: F401
     NestedStatusSerializer,
     NestedTagSerializer,
     NestedWebhookSerializer,
+    NestedJobHookSerializer,
 )
 
 #
-# Mixins
+# Mixins and Base Classes
 #
 
 
+class NotesSerializerMixin(BaseModelSerializer):
+    """Extend Serializer with a `notes` field."""
+
+    notes_url = serializers.SerializerMethodField()
+
+    def get_field_names(self, declared_fields, info):
+        """Ensure that fields includes "notes_url" field if applicable."""
+        fields = list(super().get_field_names(declared_fields, info))
+        if hasattr(self.Meta.model, "notes"):
+            self.extend_field_names(fields, "notes_url")
+        return fields
+
+    @extend_schema_field(serializers.URLField())
+    def get_notes_url(self, instance):
+        notes_url = f"{instance._meta.app_label}-api:{instance._meta.model_name}-notes"
+        if instance._meta.app_label in settings.PLUGINS:
+            notes_url = f"plugins-api:{notes_url}"
+        return reverse(notes_url, args=[instance.id], request=self.context["request"])
+
+
+class NautobotModelSerializer(
+    RelationshipModelSerializerMixin, CustomFieldModelSerializer, NotesSerializerMixin, ValidatedModelSerializer
+):
+    """Base class to use for serializers based on OrganizationalModel or PrimaryModel.
+
+    Can also be used for models derived from BaseModel, so long as they support custom fields and relationships.
+    """
+
+
+class StatusModelSerializerMixin(BaseModelSerializer):
+    """Mixin to add `status` choice field to model serializers."""
+
+    status = StatusSerializerField(queryset=Status.objects.all())
+
+    def get_field_names(self, declared_fields, info):
+        """Ensure that "status" field is always present."""
+        fields = list(super().get_field_names(declared_fields, info))
+        self.extend_field_names(fields, "status")
+        return fields
+
+    @classproperty
+    def status_choices(cls):
+        """
+        Get the list of valid status values for this serializer.
+
+        In the case where multiple serializers have the same set of status choices, it's necessary to set
+        settings.SPECTACULAR_SETTINGS["ENUM_NAME_OVERRIDES"] for at least one of the matching serializers,
+        or else drf-spectacular will report:
+        'enum naming encountered a non-optimally resolvable collision for fields named "status"'
+        """
+        return list(cls().fields["status"].get_choices().keys())
+
+
 class TagSerializerField(NestedTagSerializer):
     """NestedSerializer field for `Tag` object fields."""
 
@@ -113,9 +179,16 @@ class TagSerializerField(NestedTagSerializer):
         return queryset.get_for_model(model)
 
 
-class TaggedObjectSerializer(serializers.Serializer):
+# TODO should be TaggedModelSerializerMixin
+class TaggedObjectSerializer(BaseModelSerializer):
     tags = TagSerializerField(many=True, required=False)
 
+    def get_field_names(self, declared_fields, info):
+        """Ensure that 'tags' field is always present."""
+        fields = list(super().get_field_names(declared_fields, info))
+        self.extend_field_names(fields, "tags")
+        return fields
+
     def create(self, validated_data):
         tags = validated_data.pop("tags", None)
         instance = super().create(validated_data)
@@ -150,7 +223,7 @@ class TaggedObjectSerializer(serializers.Serializer):
 #
 
 
-class ComputedFieldSerializer(ValidatedModelSerializer):
+class ComputedFieldSerializer(ValidatedModelSerializer, NotesSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:computedfield-detail")
     content_type = ContentTypeField(
         queryset=ContentType.objects.filter(FeatureQuery("custom_fields").get_query()).order_by("app_label", "model"),
@@ -159,7 +232,6 @@ class ComputedFieldSerializer(ValidatedModelSerializer):
     class Meta:
         model = ComputedField
         fields = (
-            "id",
             "url",
             "slug",
             "label",
@@ -176,7 +248,7 @@ class ComputedFieldSerializer(ValidatedModelSerializer):
 #
 
 
-class ConfigContextSerializer(ValidatedModelSerializer):
+class ConfigContextSerializer(ValidatedModelSerializer, NotesSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:configcontext-detail")
     owner_content_type = ContentTypeField(
         queryset=ContentType.objects.filter(FeatureQuery("config_context_owners").get_query()),
@@ -198,6 +270,12 @@ class ConfigContextSerializer(ValidatedModelSerializer):
         required=False,
         many=True,
     )
+    locations = SerializedPKRelatedField(
+        queryset=Location.objects.all(),
+        serializer=NestedLocationSerializer,
+        required=False,
+        many=True,
+    )
     roles = SerializedPKRelatedField(
         queryset=DeviceRole.objects.all(),
         serializer=NestedDeviceRoleSerializer,
@@ -245,7 +323,6 @@ class ConfigContextSerializer(ValidatedModelSerializer):
     class Meta:
         model = ConfigContext
         fields = [
-            "id",
             "url",
             "name",
             "owner_content_type",
@@ -257,6 +334,7 @@ class ConfigContextSerializer(ValidatedModelSerializer):
             "is_active",
             "regions",
             "sites",
+            "locations",
             "roles",
             "device_types",
             "platforms",
@@ -266,8 +344,6 @@ class ConfigContextSerializer(ValidatedModelSerializer):
             "tenants",
             "tags",
             "data",
-            "created",
-            "last_updated",
         ]
 
     @extend_schema_field(serializers.DictField(allow_null=True))
@@ -284,7 +360,7 @@ class ConfigContextSerializer(ValidatedModelSerializer):
 #
 
 
-class ConfigContextSchemaSerializer(ValidatedModelSerializer):
+class ConfigContextSchemaSerializer(NautobotModelSerializer):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:configcontextschema-detail")
     owner_content_type = ContentTypeField(
         queryset=ContentType.objects.filter(FeatureQuery("config_context_owners").get_query()),
@@ -297,7 +373,6 @@ class ConfigContextSchemaSerializer(ValidatedModelSerializer):
     class Meta:
         model = ConfigContextSchema
         fields = [
-            "id",
             "url",
             "name",
             "slug",
@@ -306,8 +381,6 @@ class ConfigContextSchemaSerializer(ValidatedModelSerializer):
             "owner",
             "description",
             "data_schema",
-            "created",
-            "last_updated",
         ]
 
     @extend_schema_field(serializers.DictField(allow_null=True))
@@ -324,13 +397,13 @@ class ConfigContextSchemaSerializer(ValidatedModelSerializer):
 #
 
 
-class ContentTypeSerializer(serializers.ModelSerializer):
+class ContentTypeSerializer(BaseModelSerializer):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:contenttype-detail")
     display = serializers.SerializerMethodField()
 
     class Meta:
         model = ContentType
-        fields = ["id", "url", "app_label", "model", "display"]
+        fields = ["url", "app_label", "model"]
 
     @extend_schema_field(serializers.CharField)
     def get_display(self, obj):
@@ -342,7 +415,7 @@ class ContentTypeSerializer(serializers.ModelSerializer):
 #
 
 
-class CustomFieldSerializer(ValidatedModelSerializer):
+class CustomFieldSerializer(ValidatedModelSerializer, NotesSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:customfield-detail")
     content_types = ContentTypeField(
         queryset=ContentType.objects.filter(FeatureQuery("custom_fields").get_query()),
@@ -350,16 +423,20 @@ class CustomFieldSerializer(ValidatedModelSerializer):
     )
     type = ChoiceField(choices=CustomFieldTypeChoices)
     filter_logic = ChoiceField(choices=CustomFieldFilterLogicChoices, required=False)
+    # Laying groundwork for 2.0:
+    # - in 1.4 `label` is not required at the model level due to backward compatibility, but in 2.0 it will be.
+    # - in 1.4 `name` is required at the model level, but in 2.0 it will be removed entirely.
+    # Since the API is versioned we can go ahead and adopt the future here.
+    label = serializers.CharField(max_length=50, required=True)
 
     class Meta:
         model = CustomField
         fields = [
-            "id",
             "url",
             "content_types",
             "type",
-            "name",
             "label",
+            "slug",
             "description",
             "required",
             "filter_logic",
@@ -370,6 +447,37 @@ class CustomFieldSerializer(ValidatedModelSerializer):
             "validation_regex",
         ]
 
+    def validate(self, data):
+        # 2.0 TODO: #824 remove `name` entirely from the model; for now it's required.
+        if self.instance is None:
+            if "slug" in data and "name" not in data:
+                data["name"] = data["slug"]
+
+        return super().validate(data)
+
+
+class CustomFieldSerializerVersion12(CustomFieldSerializer):
+    # In older versions of the REST API, neither `label` nor `slug` were required fields. See also validate() below.
+    label = serializers.CharField(max_length=50, required=False)
+    slug = serializers.CharField(max_length=50, required=False)
+
+    class Meta(CustomFieldSerializer.Meta):
+        fields = CustomFieldSerializer.Meta.fields.copy()
+        fields.insert(4, "name")
+
+    def validate(self, data):
+        # Logic copied from CustomField.clean_fields(), since this needs to happen *before* the instance is created
+        if self.instance is None:
+            # 2.0 TODO: this is to fix up existing usage when caller specifies a name but not a label;
+            # in 2.0 we should make `label` a mandatory field when getting rid of `name`.
+            if "name" in data and "label" not in data:
+                data["label"] = data["name"]
+
+            if "label" in data and "slug" not in data:
+                data["slug"] = slugify_dashes_to_underscores(data["label"])
+
+        return super().validate(data)
+
 
 class CustomFieldChoiceSerializer(ValidatedModelSerializer):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:customfieldchoice-detail")
@@ -377,7 +485,7 @@ class CustomFieldChoiceSerializer(ValidatedModelSerializer):
 
     class Meta:
         model = CustomFieldChoice
-        fields = ["id", "url", "field", "value", "weight"]
+        fields = ["url", "field", "value", "weight"]
 
 
 #
@@ -385,7 +493,7 @@ class CustomFieldChoiceSerializer(ValidatedModelSerializer):
 #
 
 
-class CustomLinkSerializer(ValidatedModelSerializer):
+class CustomLinkSerializer(ValidatedModelSerializer, NotesSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:customlink-detail")
     content_type = ContentTypeField(
         queryset=ContentType.objects.filter(FeatureQuery("custom_links").get_query()).order_by("app_label", "model"),
@@ -394,7 +502,6 @@ class CustomLinkSerializer(ValidatedModelSerializer):
     class Meta:
         model = CustomLink
         fields = (
-            "id",
             "url",
             "target_url",
             "name",
@@ -412,32 +519,45 @@ class CustomLinkSerializer(ValidatedModelSerializer):
 #
 
 
-class DynamicGroupSerializer(ValidatedModelSerializer):
+class DynamicGroupSerializer(NautobotModelSerializer):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:dynamicgroup-detail")
     content_type = ContentTypeField(
         queryset=ContentType.objects.filter(FeatureQuery("dynamic_groups").get_query()).order_by("app_label", "model"),
     )
+    # Read-only because m2m is hard. Easier to just create # `DynamicGroupMemberships` explicitly
+    # using their own endpoint at /api/extras/dynamic-group-memberships/.
+    children = NestedDynamicGroupMembershipSerializer(source="dynamic_group_memberships", read_only=True, many=True)
 
     class Meta:
         model = DynamicGroup
         fields = [
-            "id",
             "url",
             "name",
             "slug",
             "description",
             "content_type",
             "filter",
+            "children",
         ]
         extra_kwargs = {"filter": {"read_only": False}}
 
 
+class DynamicGroupMembershipSerializer(ValidatedModelSerializer):
+    url = serializers.HyperlinkedIdentityField(view_name="extras-api:dynamicgroupmembership-detail")
+    group = NestedDynamicGroupSerializer()
+    parent_group = NestedDynamicGroupSerializer()
+
+    class Meta:
+        model = DynamicGroupMembership
+        fields = ["url", "group", "parent_group", "operator", "weight"]
+
+
 #
 # Export templates
 #
 
-
-class ExportTemplateSerializer(ValidatedModelSerializer):
+# TODO: export-templates don't support custom-fields, is this omission intentional?
+class ExportTemplateSerializer(RelationshipModelSerializerMixin, ValidatedModelSerializer, NotesSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:exporttemplate-detail")
     content_type = ContentTypeField(
         queryset=ContentType.objects.filter(FeatureQuery("export_templates").get_query()),
@@ -453,7 +573,6 @@ class ExportTemplateSerializer(ValidatedModelSerializer):
     class Meta:
         model = ExportTemplate
         fields = [
-            "id",
             "url",
             "content_type",
             "owner_content_type",
@@ -480,7 +599,7 @@ class ExportTemplateSerializer(ValidatedModelSerializer):
 #
 
 
-class GitRepositorySerializer(CustomFieldModelSerializer):
+class GitRepositorySerializer(NautobotModelSerializer):
     """Git repositories defined as a data source."""
 
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:gitrepository-detail")
@@ -497,7 +616,6 @@ class GitRepositorySerializer(CustomFieldModelSerializer):
     class Meta:
         model = GitRepository
         fields = [
-            "id",
             "url",
             "name",
             "slug",
@@ -508,12 +626,7 @@ class GitRepositorySerializer(CustomFieldModelSerializer):
             "secrets_group",
             "current_head",
             "provided_contents",
-            "created",
-            "last_updated",
-            "custom_fields",
-            "computed_fields",
         ]
-        opt_in_fields = ["computed_fields"]
 
     def validate(self, data):
         """
@@ -528,14 +641,13 @@ class GitRepositorySerializer(CustomFieldModelSerializer):
 #
 
 
-class GraphQLQuerySerializer(ValidatedModelSerializer):
+class GraphQLQuerySerializer(ValidatedModelSerializer, NotesSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:graphqlquery-detail")
     variables = serializers.DictField(required=False, allow_null=True, default={})
 
     class Meta:
         model = GraphQLQuery
         fields = (
-            "id",
             "url",
             "name",
             "slug",
@@ -565,7 +677,6 @@ class ImageAttachmentSerializer(ValidatedModelSerializer):
     class Meta:
         model = ImageAttachment
         fields = [
-            "id",
             "url",
             "content_type",
             "object_id",
@@ -598,6 +709,8 @@ class ImageAttachmentSerializer(ValidatedModelSerializer):
         # Static mapping of models to their nested serializers
         if isinstance(obj.parent, Device):
             serializer = NestedDeviceSerializer
+        elif isinstance(obj.parent, Location):
+            serializer = NestedLocationSerializer
         elif isinstance(obj.parent, Rack):
             serializer = NestedRackSerializer
         elif isinstance(obj.parent, Site):
@@ -613,13 +726,12 @@ class ImageAttachmentSerializer(ValidatedModelSerializer):
 #
 
 
-class JobSerializer(TaggedObjectSerializer, CustomFieldModelSerializer):
+class JobSerializer(NautobotModelSerializer, TaggedObjectSerializer):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:job-detail")
 
     class Meta:
         model = Job
         fields = [
-            "id",
             "url",
             "source",
             "module_name",
@@ -633,6 +745,9 @@ class JobSerializer(TaggedObjectSerializer, CustomFieldModelSerializer):
             "description_override",
             "installed",
             "enabled",
+            "is_job_hook_receiver",
+            "has_sensitive_variables",
+            "has_sensitive_variables_override",
             "approval_required",
             "approval_required_override",
             "commit_default",
@@ -646,12 +761,26 @@ class JobSerializer(TaggedObjectSerializer, CustomFieldModelSerializer):
             "time_limit",
             "time_limit_override",
             "tags",
-            "custom_fields",
-            "created",
-            "last_updated",
-            "computed_fields",
         ]
-        opt_in_fields = ["computed_fields"]
+
+    def validate(self, data):
+        # note no validation for on creation of jobs because we do not support user creation of Job records via API
+        if self.instance:
+            has_sensitive_variables = data.get("has_sensitive_variables", self.instance.has_sensitive_variables)
+            approval_required = data.get("approval_required", self.instance.approval_required)
+
+            if approval_required and has_sensitive_variables:
+                error_message = "A job with sensitive variables cannot also be marked as requiring approval"
+                errors = {}
+
+                if "approval_required" in data:
+                    errors["approval_required"] = [error_message]
+                if "has_sensitive_variables" in data:
+                    errors["has_sensitive_variables"] = [error_message]
+
+                raise serializers.ValidationError(errors)
+
+        return super().validate(data)
 
 
 class JobVariableSerializer(serializers.Serializer):
@@ -684,7 +813,7 @@ class JobRunResponseSerializer(serializers.Serializer):
 #
 
 
-class JobResultSerializer(serializers.ModelSerializer):
+class JobResultSerializer(CustomFieldModelSerializer, BaseModelSerializer):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:jobresult-detail")
     user = NestedUserSerializer(read_only=True)
     status = ChoiceField(choices=JobResultStatusChoices, read_only=True)
@@ -695,7 +824,6 @@ class JobResultSerializer(serializers.ModelSerializer):
     class Meta:
         model = JobResult
         fields = [
-            "id",
             "url",
             "created",
             "completed",
@@ -706,6 +834,7 @@ class JobResultSerializer(serializers.ModelSerializer):
             "user",
             "data",
             "job_id",
+            "job_kwargs",
             "schedule",
         ]
 
@@ -715,7 +844,7 @@ class JobResultSerializer(serializers.ModelSerializer):
 #
 
 
-class ScheduledJobSerializer(serializers.ModelSerializer):
+class ScheduledJobSerializer(BaseModelSerializer):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:scheduledjob-detail")
     user = NestedUserSerializer(read_only=True)
     job_model = NestedJobSerializer(read_only=True)
@@ -724,7 +853,6 @@ class ScheduledJobSerializer(serializers.ModelSerializer):
     class Meta:
         model = ScheduledJob
         fields = [
-            "id",
             "url",
             "name",
             "user",
@@ -741,6 +869,7 @@ class ScheduledJobSerializer(serializers.ModelSerializer):
             "approved_by_user",
             "approval_required",
             "approved_at",
+            "crontab",
         ]
 
 
@@ -784,19 +913,58 @@ class JobClassDetailSerializer(JobClassSerializer):
     result = JobResultSerializer(required=False)
 
 
+class JobHookSerializer(NautobotModelSerializer):
+    url = serializers.HyperlinkedIdentityField(view_name="extras-api:jobhook-detail")
+    content_types = ContentTypeField(
+        queryset=ChangeLoggedModelsQuery().as_queryset(),
+        many=True,
+    )
+
+    class Meta:
+        model = JobHook
+        fields = [
+            "id",
+            "url",
+            "name",
+            "content_types",
+            "job",
+            "enabled",
+            "type_create",
+            "type_update",
+            "type_delete",
+        ]
+
+    def validate(self, data):
+        validated_data = super().validate(data)
+
+        conflicts = JobHook.check_for_conflicts(
+            instance=self.instance,
+            content_types=data.get("content_types"),
+            job=data.get("job"),
+            type_create=data.get("type_create"),
+            type_update=data.get("type_update"),
+            type_delete=data.get("type_delete"),
+        )
+
+        if conflicts:
+            raise serializers.ValidationError(conflicts)
+
+        return validated_data
+
+
 class JobInputSerializer(serializers.Serializer):
-    data = serializers.JSONField(required=False, default="")
+    data = serializers.JSONField(required=False, default=dict)
     commit = serializers.BooleanField(required=False, default=None)
     schedule = NestedScheduledJobSerializer(required=False)
 
 
-class JobLogEntrySerializer(serializers.ModelSerializer):
+class JobLogEntrySerializer(BaseModelSerializer):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:joblogentry-detail")
+    display = serializers.SerializerMethodField()
 
     class Meta:
         model = JobLogEntry
         fields = [
-            "id",
             "url",
             "absolute_url",
             "created",
@@ -807,13 +975,57 @@ class JobLogEntrySerializer(serializers.ModelSerializer):
             "message",
         ]
 
+    @extend_schema_field(serializers.CharField)
+    def get_display(self, obj):
+        return obj.created.isoformat()
+
+
+#
+# Notes
+#
+
+
+class NoteSerializer(BaseModelSerializer):
+    url = serializers.HyperlinkedIdentityField(view_name="extras-api:note-detail")
+    user = NestedUserSerializer(read_only=True)
+    assigned_object_type = ContentTypeField(queryset=ContentType.objects.all())
+    assigned_object = serializers.SerializerMethodField()
+
+    class Meta:
+        model = Note
+        fields = [
+            "url",
+            "user",
+            "user_name",
+            "assigned_object_type",
+            "assigned_object_id",
+            "assigned_object",
+            "note",
+            "slug",
+        ]
+
+    @extend_schema_field(serializers.DictField(allow_null=True))
+    def get_assigned_object(self, obj):
+        if obj.assigned_object is None:
+            return None
+        try:
+            serializer = get_serializer_for_model(obj.assigned_object, prefix="Nested")
+            context = {"request": self.context["request"]}
+            return serializer(obj.assigned_object, context=context).data
+        except SerializerNotFound:
+            return None
+
+
+class NoteInputSerializer(serializers.Serializer):
+    note = serializers.CharField()
+
 
 #
 # Change logging
 #
 
 
-class ObjectChangeSerializer(serializers.ModelSerializer):
+class ObjectChangeSerializer(BaseModelSerializer):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:objectchange-detail")
     user = NestedUserSerializer(read_only=True)
     action = ChoiceField(choices=ObjectChangeActionChoices, read_only=True)
@@ -823,7 +1035,6 @@ class ObjectChangeSerializer(serializers.ModelSerializer):
     class Meta:
         model = ObjectChange
         fields = [
-            "id",
             "url",
             "time",
             "user",
@@ -859,7 +1070,7 @@ class ObjectChangeSerializer(serializers.ModelSerializer):
 #
 
 
-class RelationshipSerializer(serializers.ModelSerializer):
+class RelationshipSerializer(ValidatedModelSerializer, NotesSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:relationship-detail")
 
     source_type = ContentTypeField(
@@ -873,7 +1084,6 @@ class RelationshipSerializer(serializers.ModelSerializer):
     class Meta:
         model = Relationship
         fields = [
-            "id",
             "url",
             "name",
             "slug",
@@ -891,6 +1101,7 @@ class RelationshipSerializer(serializers.ModelSerializer):
 
 
 class RelationshipAssociationSerializer(ValidatedModelSerializer):
+    url = serializers.HyperlinkedIdentityField(view_name="extras-api:relationshipassociation-detail")
 
     source_type = ContentTypeField(
         queryset=ContentType.objects.filter(FeatureQuery("relationships").get_query()),
@@ -905,7 +1116,7 @@ class RelationshipAssociationSerializer(ValidatedModelSerializer):
     class Meta:
         model = RelationshipAssociation
         fields = [
-            "id",
+            "url",
             "relationship",
             "source_type",
             "source_id",
@@ -919,7 +1130,7 @@ class RelationshipAssociationSerializer(ValidatedModelSerializer):
 #
 
 
-class SecretSerializer(TaggedObjectSerializer, CustomFieldModelSerializer):
+class SecretSerializer(NautobotModelSerializer, TaggedObjectSerializer):
     """Serializer for `Secret` objects."""
 
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:secret-detail")
@@ -927,23 +1138,16 @@ class SecretSerializer(TaggedObjectSerializer, CustomFieldModelSerializer):
     class Meta:
         model = Secret
         fields = [
-            "id",
             "url",
             "name",
             "slug",
             "description",
             "provider",
             "parameters",
-            "tags",
-            "custom_fields",
-            "created",
-            "last_updated",
-            "computed_fields",
         ]
-        opt_in_fields = ["computed_fields"]
 
 
-class SecretsGroupSerializer(CustomFieldModelSerializer):
+class SecretsGroupSerializer(NautobotModelSerializer):
     """Serializer for `SecretsGroup` objects."""
 
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:secretsgroup-detail")
@@ -958,21 +1162,15 @@ class SecretsGroupSerializer(CustomFieldModelSerializer):
     class Meta:
         model = SecretsGroup
         fields = [
-            "id",
             "url",
             "name",
             "slug",
             "description",
             "secrets",
-            "custom_fields",
-            "created",
-            "last_updated",
-            "computed_fields",
         ]
-        opt_in_fields = ["computed_fields"]
 
 
-class SecretsGroupAssociationSerializer(BaseModelSerializer):
+class SecretsGroupAssociationSerializer(ValidatedModelSerializer):
     """Serializer for `SecretsGroupAssociation` objects."""
 
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:secretsgroupassociation-detail")
@@ -982,7 +1180,6 @@ class SecretsGroupAssociationSerializer(BaseModelSerializer):
     class Meta:
         model = SecretsGroupAssociation
         fields = [
-            "id",
             "url",
             "group",
             "access_type",
@@ -996,7 +1193,7 @@ class SecretsGroupAssociationSerializer(BaseModelSerializer):
 #
 
 
-class StatusSerializer(CustomFieldModelSerializer):
+class StatusSerializer(NautobotModelSerializer):
     """Serializer for `Status` objects."""
 
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:status-detail")
@@ -1008,68 +1205,44 @@ class StatusSerializer(CustomFieldModelSerializer):
     class Meta:
         model = Status
         fields = [
-            "id",
             "url",
             "content_types",
             "name",
             "slug",
             "color",
-            "custom_fields",
-            "created",
-            "last_updated",
         ]
 
 
-class StatusModelSerializerMixin(serializers.Serializer):
-    """Mixin to add `status` choice field to model serializers."""
-
-    status = StatusSerializerField(queryset=Status.objects.all())
-
-    @classproperty
-    def status_choices(cls):
-        """
-        Get the list of valid status values for this serializer.
-
-        May be necessary to use with settings.SPECTACULAR_SETTINGS["ENUM_NAME_OVERRIDES"] at some point if
-        we ever end up with multiple serializers whose default set of status choices are identical.
-        """
-        return list(cls().fields["status"].get_choices().keys())
-
-
 #
 # Tags
 #
 
 
-class TagSerializer(CustomFieldModelSerializer):
+class TagSerializer(NautobotModelSerializer):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:tag-detail")
     tagged_items = serializers.IntegerField(read_only=True)
 
     class Meta:
         model = Tag
         fields = [
-            "id",
             "url",
             "name",
             "slug",
             "color",
             "description",
             "tagged_items",
-            "custom_fields",
-            "created",
-            "last_updated",
         ]
 
     def validate(self, data):
         data = super().validate(data)
 
-        # All relevant content_types should be assigned to tag for API Version <1.3
-        if not data.get("content_types"):
+        # All relevant content_types should be assigned to newly created tag for API Version <1.3
+        if (self.instance is None or not self.instance.present_in_database) and "content_types" not in data:
             data["content_types"] = TaggableClassesQuery().as_queryset
 
         # check if tag is assigned to any of the removed content_types
-        if self.instance is not None and self.instance.present_in_database:
-            content_types_id = [content_type.id for content_type in data.get("content_types")]
+        if self.instance is not None and self.instance.present_in_database and "content_types" in data:
+            content_types_id = [content_type.id for content_type in data["content_types"]]
             errors = self.instance.validate_content_types_removal(content_types_id)
 
             if errors:
@@ -1088,7 +1261,6 @@ class TagSerializerVersion13(TagSerializer):
     class Meta:
         model = Tag
         fields = [
-            "id",
             "url",
             "name",
             "slug",
@@ -1096,9 +1268,6 @@ class TagSerializerVersion13(TagSerializer):
             "description",
             "tagged_items",
             "content_types",
-            "custom_fields",
-            "created",
-            "last_updated",
         ]
 
 
@@ -1107,7 +1276,7 @@ class TagSerializerVersion13(TagSerializer):
 #
 
 
-class WebhookSerializer(ValidatedModelSerializer):
+class WebhookSerializer(ValidatedModelSerializer, NotesSerializerMixin):
     url = serializers.HyperlinkedIdentityField(view_name="extras-api:webhook-detail")
     content_types = ContentTypeField(
         queryset=ContentType.objects.filter(FeatureQuery("webhooks").get_query()).order_by("app_label", "model"),
@@ -1117,7 +1286,6 @@ class WebhookSerializer(ValidatedModelSerializer):
     class Meta:
         model = Webhook
         fields = [
-            "id",
             "url",
             "content_types",
             "name",
