@@ -1,4 +1,4 @@
-from datetime import datetime
+from datetime import timedelta
 from django.contrib.contenttypes.models import ContentType
 from django.forms import ValidationError as FormsValidationError
 from django.http import Http404
@@ -10,12 +10,13 @@ from graphene_django.views import GraphQLView
 from graphql import GraphQLError
 from rest_framework import status
 from rest_framework.decorators import action
-from rest_framework.exceptions import MethodNotAllowed, PermissionDenied
+from rest_framework.exceptions import MethodNotAllowed, PermissionDenied, ValidationError
 from rest_framework.response import Response
 from rest_framework.routers import APIRootView
 from rest_framework import mixins, viewsets
 
 from nautobot.core.api.authentication import TokenPermissions
+from nautobot.core.api.filter_backends import NautobotFilterBackend
 from nautobot.core.api.metadata import ContentTypeMetadata, StatusFieldMetadata
 from nautobot.core.api.views import (
     BulkDestroyModelMixin,
@@ -32,14 +33,17 @@ from nautobot.extras.models import (
     ConfigContext,
     ConfigContextSchema,
     DynamicGroup,
+    DynamicGroupMembership,
     CustomLink,
     ExportTemplate,
     GitRepository,
     GraphQLQuery,
     ImageAttachment,
     Job,
+    JobHook,
     JobLogEntry,
     JobResult,
+    Note,
     ObjectChange,
     Relationship,
     RelationshipAssociation,
@@ -57,7 +61,12 @@ from nautobot.extras.jobs import run_job
 from nautobot.extras.utils import get_job_content_type, get_worker_count
 from nautobot.utilities.exceptions import CeleryWorkerNotRunningException
 from nautobot.utilities.api import get_serializer_for_model
-from nautobot.utilities.utils import copy_safe_request, count_related
+from nautobot.utilities.utils import (
+    copy_safe_request,
+    count_related,
+    SerializerForAPIVersions,
+    versioned_serializer_selector,
+)
 from . import nested_serializers, serializers
 
 
@@ -70,12 +79,44 @@ class ExtrasRootView(APIRootView):
         return "Extras"
 
 
+class NotesViewSetMixin:
+    @extend_schema(methods=["get"], filters=False, responses={200: serializers.NoteSerializer(many=True)})
+    @extend_schema(
+        methods=["post"],
+        request=serializers.NoteInputSerializer,
+        responses={201: serializers.NoteSerializer(many=False)},
+    )
+    @action(detail=True, url_path="notes", methods=["get", "post"])
+    def notes(self, request, pk=None):
+        """
+        API methods for returning or creating notes on an object.
+        """
+        obj = get_object_or_404(self.queryset, pk=pk)
+        if request.method == "POST":
+            content_type = ContentType.objects.get_for_model(obj)
+            data = request.data
+            data["assigned_object_id"] = obj.pk
+            data["assigned_object_type"] = f"{content_type.app_label}.{content_type.model}"
+            serializer = serializers.NoteSerializer(data=data, context={"request": request})
+
+            # Create the new Note.
+            serializer.is_valid(raise_exception=True)
+            serializer.save(user=request.user)
+            return Response(serializer.data, status=status.HTTP_201_CREATED)
+
+        else:
+            notes = self.paginate_queryset(obj.notes)
+            serializer = serializers.NoteSerializer(notes, many=True, context={"request": request})
+
+        return self.get_paginated_response(serializer.data)
+
+
 #
 #  Computed Fields
 #
 
 
-class ComputedFieldViewSet(ModelViewSet):
+class ComputedFieldViewSet(ModelViewSet, NotesViewSetMixin):
     """
     Manage Computed Fields through DELETE, GET, POST, PUT, and PATCH requests.
     """
@@ -90,6 +131,22 @@ class ComputedFieldViewSet(ModelViewSet):
 #
 
 
+class ConfigContextFilterBackend(NautobotFilterBackend):
+    """
+    Used by views that work with config context models (device and virtual machine).
+
+    Recognizes that "exclude" is not a filterset parameter but rather a view parameter (see ConfigContextQuerySetMixin)
+    """
+
+    def get_filterset_kwargs(self, request, queryset, view):
+        kwargs = super().get_filterset_kwargs(request, queryset, view)
+        try:
+            kwargs["data"].pop("exclude")
+        except KeyError:
+            pass
+        return kwargs
+
+
 class ConfigContextQuerySetMixin:
     """
     Used by views that work with config context models (device and virtual machine).
@@ -97,6 +154,8 @@ class ConfigContextQuerySetMixin:
     data annotation or not.
     """
 
+    filter_backends = [ConfigContextFilterBackend]
+
     def get_queryset(self):
         """
         Build the proper queryset based on the request context
@@ -113,7 +172,7 @@ class ConfigContextQuerySetMixin:
         return queryset.annotate_config_context_data()
 
 
-class ConfigContextViewSet(ModelViewSet):
+class ConfigContextViewSet(ModelViewSet, NotesViewSetMixin):
     queryset = ConfigContext.objects.prefetch_related(
         "regions",
         "sites",
@@ -132,7 +191,7 @@ class ConfigContextViewSet(ModelViewSet):
 #
 
 
-class ConfigContextSchemaViewSet(ModelViewSet):
+class ConfigContextSchemaViewSet(ModelViewSet, NotesViewSetMixin):
     queryset = ConfigContextSchema.objects.all()
     serializer_class = serializers.ConfigContextSchemaSerializer
     filterset_class = filters.ConfigContextSchemaFilterSet
@@ -158,12 +217,55 @@ class ContentTypeViewSet(viewsets.ReadOnlyModelViewSet):
 #
 
 
-class CustomFieldViewSet(ModelViewSet):
+@extend_schema_view(
+    bulk_partial_update=extend_schema(
+        filters=False,
+        request=serializers.CustomFieldSerializerVersion12(many=True),
+        responses={"200": serializers.CustomFieldSerializerVersion12(many=True)},
+        versions=["1.2", "1.3"],
+    ),
+    bulk_update=extend_schema(
+        filters=False,
+        request=serializers.CustomFieldSerializerVersion12(many=True),
+        responses={"200": serializers.CustomFieldSerializerVersion12(many=True)},
+        versions=["1.2", "1.3"],
+    ),
+    create=extend_schema(
+        request=serializers.CustomFieldSerializerVersion12,
+        responses={"201": serializers.CustomFieldSerializerVersion12},
+        versions=["1.2", "1.3"],
+    ),
+    list=extend_schema(
+        responses={"200": serializers.CustomFieldSerializerVersion12(many=True)}, versions=["1.2", "1.3"]
+    ),
+    partial_update=extend_schema(
+        request=serializers.CustomFieldSerializerVersion12,
+        responses={"200": serializers.CustomFieldSerializerVersion12},
+        versions=["1.2", "1.3"],
+    ),
+    retrieve=extend_schema(responses={"200": serializers.CustomFieldSerializerVersion12}, versions=["1.2", "1.3"]),
+    update=extend_schema(
+        request=serializers.CustomFieldSerializerVersion12,
+        responses={"200": serializers.CustomFieldSerializerVersion12},
+        versions=["1.2", "1.3"],
+    ),
+)
+class CustomFieldViewSet(ModelViewSet, NotesViewSetMixin):
     metadata_class = ContentTypeMetadata
     queryset = CustomField.objects.all()
     serializer_class = serializers.CustomFieldSerializer
     filterset_class = filters.CustomFieldFilterSet
 
+    def get_serializer_class(self):
+        serializer_choices = (
+            SerializerForAPIVersions(versions=["1.2", "1.3"], serializer=serializers.CustomFieldSerializerVersion12),
+        )
+        return versioned_serializer_selector(
+            obj=self,
+            serializer_choices=serializer_choices,
+            default_serializer=super().get_serializer_class(),
+        )
+
 
 class CustomFieldChoiceViewSet(ModelViewSet):
     queryset = CustomFieldChoice.objects.all()
@@ -191,12 +293,19 @@ class CustomFieldModelViewSet(ModelViewSet):
         return context
 
 
+class NautobotModelViewSet(CustomFieldModelViewSet, NotesViewSetMixin):
+    """Base class to use for API ViewSets based on OrganizationalModel or PrimaryModel.
+
+    Can also be used for models derived from BaseModel, so long as they support Notes.
+    """
+
+
 #
 # Custom Links
 #
 
 
-class CustomLinkViewSet(ModelViewSet):
+class CustomLinkViewSet(ModelViewSet, NotesViewSetMixin):
     """
     Manage Custom Links through DELETE, GET, POST, PUT, and PATCH requests.
     """
@@ -211,7 +320,7 @@ class CustomLinkViewSet(ModelViewSet):
 #
 
 
-class DynamicGroupViewSet(ModelViewSet):
+class DynamicGroupViewSet(ModelViewSet, NotesViewSetMixin):
     """
     Manage Dynamic Groups through DELETE, GET, POST, PUT, and PATCH requests.
     """
@@ -236,12 +345,22 @@ class DynamicGroupViewSet(ModelViewSet):
         return self.get_paginated_response(member_serializer.data)
 
 
+class DynamicGroupMembershipViewSet(ModelViewSet):
+    """
+    Manage Dynamic Group Memberships through DELETE, GET, POST, PUT, and PATCH requests.
+    """
+
+    queryset = DynamicGroupMembership.objects.prefetch_related("group", "parent_group")
+    serializer_class = serializers.DynamicGroupMembershipSerializer
+    filterset_class = filters.DynamicGroupMembershipFilterSet
+
+
 #
 # Export templates
 #
 
 
-class ExportTemplateViewSet(ModelViewSet):
+class ExportTemplateViewSet(ModelViewSet, NotesViewSetMixin):
     metadata_class = ContentTypeMetadata
     queryset = ExportTemplate.objects.all()
     serializer_class = serializers.ExportTemplateSerializer
@@ -253,7 +372,7 @@ class ExportTemplateViewSet(ModelViewSet):
 #
 
 
-class GitRepositoryViewSet(CustomFieldModelViewSet):
+class GitRepositoryViewSet(NautobotModelViewSet):
     """
     Manage the use of Git repositories as external data sources.
     """
@@ -284,7 +403,7 @@ class GitRepositoryViewSet(CustomFieldModelViewSet):
 #
 
 
-class GraphQLQueryViewSet(ModelViewSet):
+class GraphQLQueryViewSet(ModelViewSet, NotesViewSetMixin):
     queryset = GraphQLQuery.objects.all()
     serializer_class = serializers.GraphQLQuerySerializer
     filterset_class = filters.GraphQLQueryFilterSet
@@ -327,7 +446,7 @@ class ImageAttachmentViewSet(ModelViewSet):
 def _create_schedule(serializer, data, commit, job, job_model, request):
     """
     This is an internal function to create a scheduled job from API data.
-    It has to handle boths once-offs (i.e. of type TYPE_FUTURE) and interval
+    It has to handle both once-offs (i.e. of type TYPE_FUTURE) and interval
     jobs.
     """
     job_kwargs = {
@@ -339,11 +458,19 @@ def _create_schedule(serializer, data, commit, job, job_model, request):
     }
     type_ = serializer["interval"]
     if type_ == JobExecutionType.TYPE_IMMEDIATELY:
-        time = datetime.now()
+        time = timezone.now()
         name = serializer.get("name") or f"{job.name} - {time}"
+    elif type_ == JobExecutionType.TYPE_CUSTOM:
+        time = serializer.get("start_time")  # doing .get("key", "default") returns None instead of "default"
+        if time is None:
+            # "start_time" is checked against models.ScheduledJob.earliest_possible_time()
+            # which returns timezone.now() + timedelta(seconds=15)
+            time = timezone.now() + timedelta(seconds=20)
+        name = serializer["name"]
     else:
         time = serializer["start_time"]
         name = serializer["name"]
+    crontab = serializer.get("crontab", "")
 
     # 2.0 TODO: To revisit this as part of a larger Jobs cleanup in 2.0.
     #
@@ -364,6 +491,7 @@ def _create_schedule(serializer, data, commit, job, job_model, request):
         one_off=(type_ == JobExecutionType.TYPE_FUTURE),
         user=request.user,
         approval_required=job_model.approval_required,
+        crontab=crontab,
     )
     scheduled_job.save()
     return scheduled_job
@@ -377,17 +505,22 @@ def _run_job(request, job_model, legacy_response=False):
         raise PermissionDenied("This job is not enabled to be run.")
     if not job_model.installed:
         raise MethodNotAllowed(request.method, detail="This job is not presently installed and cannot be run")
+    if job_model.has_sensitive_variables:
+        if request.data.get("schedule") and request.data["schedule"]["interval"] != JobExecutionType.TYPE_IMMEDIATELY:
+            raise ValidationError(
+                {"schedule": {"interval": ["Unable to schedule job: Job may have sensitive input variables"]}}
+            )
 
     job_class = job_model.job_class
     if job_class is None:
         raise MethodNotAllowed(request.method, detail="This job's source code could not be located and cannot be run")
     job = job_class()
 
-    input_serializer = serializers.JobInputSerializer(data=request.data)
+    input_serializer = serializers.JobInputSerializer(data=request.data, context={"request": request})
     input_serializer.is_valid(raise_exception=True)
 
-    data = input_serializer.data["data"] or {}
-    commit = input_serializer.data["commit"]
+    data = input_serializer.validated_data.get("data", {})
+    commit = input_serializer.validated_data.get("commit", None)
     if commit is None:
         commit = job_model.commit_default
 
@@ -403,7 +536,7 @@ def _run_job(request, job_model, legacy_response=False):
         raise CeleryWorkerNotRunningException()
 
     job_content_type = get_job_content_type()
-    schedule_data = input_serializer.data.get("schedule")
+    schedule_data = input_serializer.validated_data.get("schedule")
 
     # Default to a null JobResult.
     job_result = None
@@ -413,6 +546,14 @@ def _run_job(request, job_model, legacy_response=False):
     if schedule_data is None and job_model.approval_required:
         schedule_data = {"interval": JobExecutionType.TYPE_IMMEDIATELY}
 
+    # Skip creating a ScheduledJob when job can be executed immediately
+    elif (
+        schedule_data
+        and schedule_data["interval"] == JobExecutionType.TYPE_IMMEDIATELY
+        and not job_model.approval_required
+    ):
+        schedule_data = None
+
     # Try to create a ScheduledJob, or...
     if schedule_data:
         schedule = _create_schedule(schedule_data, data, commit, job, job_model, request)
@@ -460,6 +601,7 @@ class JobViewSet(
     BulkDestroyModelMixin,
     # Base class
     ReadOnlyModelViewSet,
+    NotesViewSetMixin,
 ):
     queryset = Job.objects.all()
     serializer_class = serializers.JobSerializer
@@ -630,6 +772,21 @@ class JobViewSet(
         return _run_job(request, job_model, legacy_response=True)
 
 
+#
+# Job Hooks
+#
+
+
+class JobHooksViewSet(NautobotModelViewSet):
+    """
+    Manage job hooks through DELETE, GET, POST, PUT, and PATCH requests.
+    """
+
+    queryset = JobHook.objects.all()
+    serializer_class = serializers.JobHookSerializer
+    filterset_class = filters.JobHookFilterSet
+
+
 #
 # Job Results
 #
@@ -645,7 +802,15 @@ class JobLogEntryViewSet(ReadOnlyModelViewSet):
     filterset_class = filters.JobLogEntryFilterSet
 
 
-class JobResultViewSet(ModelViewSet):
+class JobResultViewSet(
+    # DRF mixins:
+    # note no CreateModelMixin or UpdateModelMixin
+    mixins.DestroyModelMixin,
+    # Nautobot mixins:
+    BulkDestroyModelMixin,
+    # Base class
+    ReadOnlyModelViewSet,
+):
     """
     Retrieve a list of job results
     """
@@ -802,6 +967,22 @@ class ScheduledJobViewSet(ReadOnlyModelViewSet):
         return Response(serializer.data)
 
 
+#
+# Notes
+#
+
+
+class NoteViewSet(ModelViewSet):
+    metadata_class = ContentTypeMetadata
+    queryset = Note.objects.prefetch_related("user")
+    serializer_class = serializers.NoteSerializer
+    filterset_class = filters.NoteFilterSet
+
+    # Assign user from request
+    def perform_create(self, serializer):
+        serializer.save(user=self.request.user)
+
+
 #
 # Change logging
 #
@@ -823,7 +1004,7 @@ class ObjectChangeViewSet(ReadOnlyModelViewSet):
 #
 
 
-class RelationshipViewSet(ModelViewSet):
+class RelationshipViewSet(ModelViewSet, NotesViewSetMixin):
     metadata_class = ContentTypeMetadata
     queryset = Relationship.objects.all()
     serializer_class = serializers.RelationshipSerializer
@@ -842,7 +1023,7 @@ class RelationshipAssociationViewSet(ModelViewSet):
 #
 
 
-class SecretsViewSet(ModelViewSet):
+class SecretsViewSet(NautobotModelViewSet):
     """
     Manage Secrets through DELETE, GET, POST, PUT, and PATCH requests.
     """
@@ -852,7 +1033,7 @@ class SecretsViewSet(ModelViewSet):
     filterset_class = filters.SecretFilterSet
 
 
-class SecretsGroupViewSet(ModelViewSet):
+class SecretsGroupViewSet(NautobotModelViewSet):
     """
     Manage Secrets Groups through DELETE, GET, POST, PUT, and PATCH requests.
     """
@@ -877,7 +1058,7 @@ class SecretsGroupAssociationViewSet(ModelViewSet):
 #
 
 
-class StatusViewSet(CustomFieldModelViewSet):
+class StatusViewSet(NautobotModelViewSet):
     """
     View and manage custom status choices for objects with a `status` field.
     """
@@ -901,31 +1082,42 @@ class StatusViewSetMixin(ModelViewSet):
 
 
 @extend_schema_view(
-    bulk_update=extend_schema(responses={"200": serializers.TagSerializer(many=True)}, versions=["1.2"]),
-    bulk_partial_update=extend_schema(responses={"200": serializers.TagSerializer(many=True)}, versions=["1.2"]),
-    create=extend_schema(responses={"201": serializers.TagSerializer}, versions=["1.2"]),
-    partial_update=extend_schema(responses={"200": serializers.TagSerializer}, versions=["1.2"]),
-    update=extend_schema(responses={"200": serializers.TagSerializer}, versions=["1.2"]),
+    bulk_update=extend_schema(
+        filters=False,
+        request=serializers.TagSerializer(many=True),
+        responses={"200": serializers.TagSerializer(many=True)},
+        versions=["1.2"],
+    ),
+    bulk_partial_update=extend_schema(
+        filters=False,
+        request=serializers.TagSerializer(many=True),
+        responses={"200": serializers.TagSerializer(many=True)},
+        versions=["1.2"],
+    ),
+    create=extend_schema(
+        request=serializers.TagSerializer, responses={"201": serializers.TagSerializer}, versions=["1.2"]
+    ),
+    partial_update=extend_schema(
+        request=serializers.TagSerializer, responses={"200": serializers.TagSerializer}, versions=["1.2"]
+    ),
+    update=extend_schema(
+        request=serializers.TagSerializer, responses={"200": serializers.TagSerializer}, versions=["1.2"]
+    ),
     list=extend_schema(responses={"200": serializers.TagSerializer(many=True)}, versions=["1.2"]),
     retrieve=extend_schema(responses={"200": serializers.TagSerializer}, versions=["1.2"]),
 )
-class TagViewSet(CustomFieldModelViewSet):
+class TagViewSet(NautobotModelViewSet):
     queryset = Tag.objects.annotate(tagged_items=count_related(TaggedItem, "tag"))
     serializer_class = serializers.TagSerializerVersion13
     filterset_class = filters.TagFilterSet
 
     def get_serializer_class(self):
-        if (
-            not self.brief
-            and not getattr(self, "swagger_fake_view", False)
-            and (
-                not hasattr(self.request, "major_version")
-                or self.request.major_version > 1
-                or (self.request.major_version == 1 and self.request.minor_version < 3)
-            )
-        ):
-            return serializers.TagSerializer
-        return super().get_serializer_class()
+        serializer_choices = (SerializerForAPIVersions(versions=["1.2"], serializer=serializers.TagSerializer),)
+        return versioned_serializer_selector(
+            obj=self,
+            serializer_choices=serializer_choices,
+            default_serializer=super().get_serializer_class(),
+        )
 
 
 #
@@ -933,7 +1125,7 @@ class TagViewSet(CustomFieldModelViewSet):
 #
 
 
-class WebhooksViewSet(ModelViewSet):
+class WebhooksViewSet(ModelViewSet, NotesViewSetMixin):
     """
     Manage Webhooks through DELETE, GET, POST, PUT, and PATCH requests.
     """
