@@ -12,7 +12,7 @@ from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ObjectDoesNotExist, ValidationError
 from django.core.serializers.json import DjangoJSONEncoder
 from django.core.validators import MinValueValidator
-from django.db import models
+from django.db import models, transaction
 from django.db.models import signals
 from django.urls import reverse
 from django.utils import timezone
@@ -23,7 +23,7 @@ from django_celery_beat.managers import ExtendedManager
 from nautobot.core.celery import NautobotKombuJSONEncoder
 from nautobot.core.fields import AutoSlugField, slugify_dots_to_dashes
 from nautobot.core.models import BaseModel
-from nautobot.core.models.generics import PrimaryModel
+from nautobot.core.models.generics import OrganizationalModel, PrimaryModel
 from nautobot.extras.choices import JobExecutionType, JobResultStatusChoices, JobSourceChoices, LogLevelChoices
 from nautobot.extras.constants import (
     JOB_LOG_MAX_ABSOLUTE_URL_LENGTH,
@@ -37,7 +37,13 @@ from nautobot.extras.constants import (
 )
 from nautobot.extras.plugins.utils import import_object
 from nautobot.extras.querysets import JobQuerySet, ScheduledJobExtendedQuerySet
-from nautobot.extras.utils import get_job_content_type, extras_features, FeatureQuery, jobs_in_directory
+from nautobot.extras.utils import (
+    ChangeLoggedModelsQuery,
+    FeatureQuery,
+    extras_features,
+    get_job_content_type,
+    jobs_in_directory,
+)
 from nautobot.utilities.logging import sanitize
 
 from .customfields import CustomFieldModel
@@ -127,6 +133,14 @@ class Job(PrimaryModel):
     )
     enabled = models.BooleanField(default=False, help_text="Whether this job can be executed by users")
 
+    is_job_hook_receiver = models.BooleanField(
+        default=False, editable=False, help_text="Whether this job is a job hook receiver"
+    )
+
+    has_sensitive_variables = models.BooleanField(
+        default=True, help_text="Whether this job contains sensitive variables"
+    )
+
     # Additional properties, potentially inherited from the source code
     # See also the docstring of nautobot.extras.jobs.BaseJob.Meta.
     approval_required = models.BooleanField(
@@ -195,6 +209,10 @@ class Job(PrimaryModel):
         default=False,
         help_text="If set, the configured value will remain even if the underlying Job source code changes",
     )
+    has_sensitive_variables_override = models.BooleanField(
+        default=False,
+        help_text="If set, the configured value will remain even if the underlying Job source code changes",
+    )
 
     objects = JobQuerySet.as_manager()
 
@@ -251,7 +269,7 @@ class Job(PrimaryModel):
                     logger.warning("Job %s %s has no associated Git repository", self.module_name, self.job_class_name)
                     return None
                 try:
-                    # In the case where we have multiple Nautobot instances, or multiple RQ worker instances,
+                    # In the case where we have multiple Nautobot instances, or multiple worker instances,
                     # they are not required to share a common filesystem; therefore, we may need to refresh our local
                     # clone of the Git repository to ensure that it is in sync with the latest repository clone
                     # from any instance.
@@ -292,7 +310,7 @@ class Job(PrimaryModel):
     @property
     def latest_result(self):
         if self._latest_result is None:
-            self._latest_result = self.results.last()
+            self._latest_result = self.results.first()
         return self._latest_result
 
     @property
@@ -327,10 +345,111 @@ class Job(PrimaryModel):
         if len(self.slug) > JOB_MAX_SLUG_LENGTH:
             raise ValidationError(f"Slug may not exceed {JOB_MAX_SLUG_LENGTH} characters in length")
 
+        if self.has_sensitive_variables is True and self.approval_required is True:
+            raise ValidationError(
+                {"approval_required": "A job that may have sensitive variables cannot be marked as requiring approval"}
+            )
+
     def get_absolute_url(self):
         return reverse("extras:job_detail", kwargs={"slug": self.slug})
 
 
+@extras_features("graphql")
+class JobHook(OrganizationalModel):
+    """
+    A job hook defines a request that will trigger a job hook receiver when an object is created, updated, and/or
+    deleted in Nautobot. Each job hook can be limited to firing only on certain actions or certain object types.
+    """
+
+    content_types = models.ManyToManyField(
+        to=ContentType,
+        related_name="job_hooks",
+        verbose_name="Object types",
+        # 2.0 TODO: standardize verbose name for ContentType fields
+        limit_choices_to=ChangeLoggedModelsQuery,
+        help_text="The object(s) to which this job hook applies.",
+    )
+    enabled = models.BooleanField(default=True)
+    job = models.ForeignKey(
+        to=Job,
+        related_name="job_hook",
+        verbose_name="Job",
+        help_text="The job that this job hook will initiate",
+        on_delete=models.CASCADE,
+        limit_choices_to={"is_job_hook_receiver": True},
+    )
+    name = models.CharField(max_length=100, unique=True)
+    slug = AutoSlugField(populate_from="name")
+    type_create = models.BooleanField(default=False, help_text="Call this job hook when a matching object is created.")
+    type_delete = models.BooleanField(default=False, help_text="Call this job hook when a matching object is deleted.")
+    type_update = models.BooleanField(default=False, help_text="Call this job hook when a matching object is updated.")
+
+    class Meta:
+        ordering = ("name",)
+
+    def __str__(self):
+        return self.name
+
+    def clean(self):
+        super().clean()
+
+        # At least one action type must be selected
+        if not self.type_create and not self.type_delete and not self.type_update:
+            raise ValidationError("You must select at least one type: create, update, and/or delete.")
+
+    def get_absolute_url(self):
+        return reverse("extras:jobhook", kwargs={"slug": self.slug})
+
+    @classmethod
+    def check_for_conflicts(
+        cls, instance=None, content_types=None, job=None, type_create=None, type_update=None, type_delete=None
+    ):
+        """
+        Helper method for enforcing uniqueness.
+
+        Don't allow two job hooks with the same content_type, same job, and any action(s) in common.
+        Called by JobHookForm.clean() and JobHookSerializer.validate()
+        """
+
+        conflicts = {}
+        job_hook_error_msg = "A job hook already exists for {action} on {content_type} to job {job}"
+
+        if instance is not None and instance.present_in_database:
+            # This is a PATCH and might not include all relevant data
+            # Therefore we get data not available from instance
+            content_types = instance.content_types.all() if content_types is None else content_types
+            type_create = instance.type_create if type_create is None else type_create
+            type_update = instance.type_update if type_update is None else type_update
+            type_delete = instance.type_delete if type_delete is None else type_delete
+
+        if content_types is not None:
+            for content_type in content_types:
+                job_hooks = cls.objects.filter(content_types__in=[content_type], job=job)
+                if instance and instance.present_in_database:
+                    job_hooks = job_hooks.exclude(pk=instance.pk)
+
+                existing_type_create = job_hooks.filter(type_create=type_create).exists() if type_create else False
+                existing_type_update = job_hooks.filter(type_update=type_update).exists() if type_update else False
+                existing_type_delete = job_hooks.filter(type_delete=type_delete).exists() if type_delete else False
+
+                if existing_type_create:
+                    conflicts.setdefault("type_create", []).append(
+                        job_hook_error_msg.format(content_type=content_type, action="create", job=job),
+                    )
+
+                if existing_type_update:
+                    conflicts.setdefault("type_update", []).append(
+                        job_hook_error_msg.format(content_type=content_type, action="update", job=job),
+                    )
+
+                if existing_type_delete:
+                    conflicts.setdefault("type_delete", []).append(
+                        job_hook_error_msg.format(content_type=content_type, action="delete", job=job),
+                    )
+
+        return conflicts
+
+
 @extras_features(
     "graphql",
 )
@@ -394,12 +513,14 @@ class JobResult(BaseModel, CustomFieldModel):
     user = models.ForeignKey(
         to=settings.AUTH_USER_MODEL, on_delete=models.SET_NULL, related_name="+", blank=True, null=True
     )
+    # todoindex:
     status = models.CharField(
         max_length=30,
         choices=JobResultStatusChoices,
         default=JobResultStatusChoices.STATUS_PENDING,
     )
     data = models.JSONField(encoder=DjangoJSONEncoder, null=True, blank=True)
+    job_kwargs = models.JSONField(blank=True, null=True, encoder=NautobotKombuJSONEncoder)
     schedule = models.ForeignKey(to="extras.ScheduledJob", on_delete=models.SET_NULL, null=True, blank=True)
     """
     Although "data" is technically an unstructured field, we have a standard structure that we try to adhere to.
@@ -543,7 +664,16 @@ class JobResult(BaseModel, CustomFieldModel):
         schedule: Optional ScheduledJob instance to link to the JobResult
         kwargs: additional kwargs passed to the callable
         """
-        job_result = cls.objects.create(name=name, obj_type=obj_type, user=user, job_id=uuid.uuid4(), schedule=schedule)
+        # Discard "request" parameter from the kwargs that we save in the job_result, as it's not relevant to re-runs,
+        # and will likely go away in the future.
+        job_result_kwargs = {key: value for key, value in kwargs.items() if key != "request"}
+        job_result = cls.objects.create(
+            name=name,
+            obj_type=obj_type,
+            user=user,
+            job_id=uuid.uuid4(),
+            schedule=schedule,
+        )
 
         kwargs["job_result_pk"] = job_result.pk
 
@@ -558,6 +688,8 @@ class JobResult(BaseModel, CustomFieldModel):
                     celery_kwargs["soft_time_limit"] = job_model.soft_time_limit
                 if job_model.time_limit > 0:
                     celery_kwargs["time_limit"] = job_model.time_limit
+                if not job_model.has_sensitive_variables:
+                    job_result.job_kwargs = job_result_kwargs
                 job_result.job_model = job_model
                 job_result.save()
             except Job.DoesNotExist:
@@ -571,10 +703,16 @@ class JobResult(BaseModel, CustomFieldModel):
                         celery_kwargs["soft_time_limit"] = job_class.Meta.soft_time_limit
                     if hasattr(job_class.Meta, "time_limit"):
                         celery_kwargs["time_limit"] = job_class.Meta.time_limit
+                    if not job_class.has_sensitive_variables:
+                        job_result.job_kwargs = job_result_kwargs
+                        job_result.save()
                 else:
                     logger.error("Neither a Job database record nor a Job source class were found for %s", name)
 
-        func.apply_async(args=args, kwargs=kwargs, task_id=str(job_result.job_id), **celery_kwargs)
+        # Jobs queued inside of a transaction need to run after the transaction completes and the JobResult is saved to the database
+        transaction.on_commit(
+            lambda: func.apply_async(args=args, kwargs=kwargs, task_id=str(job_result.job_id), **celery_kwargs)
+        )
 
         return job_result
 
@@ -709,6 +847,7 @@ class ScheduledJob(BaseModel):
         verbose_name="Start Datetime",
         help_text="Datetime when the schedule should begin triggering the task to run",
     )
+    # todoindex:
     enabled = models.BooleanField(
         default=True,
         verbose_name="Enabled",
@@ -754,6 +893,7 @@ class ScheduledJob(BaseModel):
         null=True,
         help_text="User that approved the schedule",
     )
+    # todoindex:
     approval_required = models.BooleanField(default=False)
     approved_at = models.DateTimeField(
         editable=False,
@@ -762,6 +902,12 @@ class ScheduledJob(BaseModel):
         verbose_name="Approval date/time",
         help_text="Datetime that the schedule was approved",
     )
+    crontab = models.CharField(
+        max_length=255,
+        blank=True,
+        verbose_name="Custom cronjob",
+        help_text="Cronjob syntax string for custom scheduling",
+    )
 
     objects = ScheduledJobExtendedQuerySet.as_manager()
     no_changes = False
@@ -776,6 +922,12 @@ class ScheduledJob(BaseModel):
         self.queue = self.queue or None
         # pass pk to worker task in kwargs, celery doesn't provide the full object to the worker
         self.kwargs["scheduled_job_pk"] = self.pk
+        # make sure non-valid crontab doesn't get saved
+        if self.interval == JobExecutionType.TYPE_CUSTOM:
+            try:
+                self.get_crontab(self.crontab)
+            except Exception as e:
+                raise ValidationError({"crontab": e})
         if not self.enabled:
             self.last_run_at = None
         elif not self.last_run_at:
@@ -817,6 +969,29 @@ class ScheduledJob(BaseModel):
     def earliest_possible_time():
         return timezone.now() + timedelta(seconds=15)
 
+    @classmethod
+    def get_crontab(cls, crontab):
+        """
+        Wrapper method translates crontab syntax to Celery crontab.
+
+        Supports following symbols:
+
+        • Asterisk (*) - signifies all possible values
+        • Comma (,) - lists multiple values
+        • Hyphen (-) - determine a range of values
+        • Slash (/) - divide a value ({*/15 * * * *} runs every 15 minutes)
+
+        No support for Last (L), Weekday (W), Number symbol (#), Question mark (?), and special @ strings.
+        """
+        minute, hour, day_of_month, month_of_year, day_of_week = crontab.split(" ")
+        return schedules.crontab(
+            minute=minute,
+            hour=hour,
+            day_of_month=day_of_month,
+            month_of_year=month_of_year,
+            day_of_week=day_of_week,
+        )
+
     def to_cron(self):
         t = self.start_time
         if self.interval == JobExecutionType.TYPE_HOURLY:
@@ -825,6 +1000,8 @@ class ScheduledJob(BaseModel):
             return schedules.crontab(minute=t.minute, hour=t.hour)
         elif self.interval == JobExecutionType.TYPE_WEEKLY:
             return schedules.crontab(minute=t.minute, hour=t.hour, day_of_week=t.weekday())
+        elif self.interval == JobExecutionType.TYPE_CUSTOM:
+            return self.get_crontab(self.crontab)
         raise ValueError(f"I do not know to convert {self.interval} to a Cronjob!")
 
 
