@@ -5,11 +5,13 @@ from django.db import models
 from django.urls import reverse
 from django.utils.encoding import force_str
 from django.utils.hashable import make_hashable
+from nautobot.extras.models.mixins import NotesMixin
 
 from nautobot.extras.utils import extras_features, FeatureQuery
 from nautobot.extras.models import ChangeLoggedModel
 from nautobot.extras.models.customfields import CustomFieldModel
 from nautobot.extras.models.relationships import RelationshipModel
+from nautobot.core.fields import AutoSlugField
 from nautobot.core.models import BaseModel
 from nautobot.utilities.querysets import RestrictedQuerySet
 from nautobot.utilities.choices import ColorChoices
@@ -27,6 +29,9 @@ class StatusQuerySet(RestrictedQuerySet):
         content_type = ContentType.objects.get_for_model(model._meta.concrete_model)
         return self.filter(content_types=content_type)
 
+    def get_by_natural_key(self, slug):
+        return self.get(slug=slug)
+
 
 @extras_features(
     "custom_fields",
@@ -37,7 +42,7 @@ class StatusQuerySet(RestrictedQuerySet):
     "relationships",
     "webhooks",
 )
-class Status(BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipModel):
+class Status(BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipModel, NotesMixin):
     """Model for database-backend enum choice objects."""
 
     content_types = models.ManyToManyField(
@@ -49,7 +54,7 @@ class Status(BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipModel):
     )
     name = models.CharField(max_length=50, unique=True)
     color = ColorField(default=ColorChoices.COLOR_GREY)
-    slug = models.SlugField(max_length=50, unique=True)
+    slug = AutoSlugField(populate_from="name", max_length=50)
     description = models.CharField(
         max_length=200,
         blank=True,
@@ -67,6 +72,9 @@ class Status(BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipModel):
     def __str__(self):
         return self.name
 
+    def natural_key(self):
+        return (self.slug,)
+
     def get_absolute_url(self):
         return reverse("extras:status", args=[self.slug])
 
