@@ -7,8 +7,9 @@ from datetime import timedelta
 
 from cacheops.signals import cache_invalidated, cache_read
 from django.contrib.contenttypes.models import ContentType
+from django.core.exceptions import ValidationError
 from django.db import transaction
-from django.db.models.signals import m2m_changed, pre_delete
+from django.db.models.signals import m2m_changed, pre_delete, pre_save
 from django.dispatch import receiver
 from django.utils import timezone
 from django_prometheus.models import model_deletes, model_inserts, model_updates
@@ -18,7 +19,7 @@ from nautobot.extras.tasks import delete_custom_field_data, provision_field
 from nautobot.extras.utils import refresh_job_model_from_job_class
 from nautobot.utilities.config import get_settings_or_config
 from .choices import JobResultStatusChoices, ObjectChangeActionChoices
-from .models import CustomField, GitRepository, JobResult, ObjectChange
+from .models import CustomField, DynamicGroup, DynamicGroupMembership, GitRepository, JobResult, ObjectChange
 from .webhooks import enqueue_webhooks
 
 logger = logging.getLogger("nautobot.extras.signals")
@@ -29,23 +30,25 @@ logger = logging.getLogger("nautobot.extras.signals")
 #
 
 
-def _get_user_if_authenticated(request, objectchange):
+def _get_user_if_authenticated(user, objectchange):
     """Return the user object associated with the request if the user is defined.
 
     If the user is not defined, log a warning to indicate that the user couldn't be retrived from the request
     This is a workaround to fix a recurring issue where the user shouldn't be present in the request object randomly.
     A similar issue was reported in NetBox https://github.com/netbox-community/netbox/issues/5142
     """
-    if request.user.is_authenticated:
-        return request.user
+    if user.is_authenticated:
+        return user
     else:
         logger.warning(f"Unable to retrieve the user while creating the changelog for {objectchange.changed_object}")
 
 
-def _handle_changed_object(request, sender, instance, **kwargs):
+def _handle_changed_object(change_context, sender, instance, **kwargs):
     """
     Fires when an object is created or updated.
     """
+    from .jobs import enqueue_job_hooks  # avoid circular import
+
     m2m_changed = False
 
     # Determine the type of change being made
@@ -63,19 +66,28 @@ def _handle_changed_object(request, sender, instance, **kwargs):
     # Record an ObjectChange if applicable
     if hasattr(instance, "to_objectchange"):
         if m2m_changed:
-            ObjectChange.objects.filter(
+            related_changes = ObjectChange.objects.filter(
                 changed_object_type=ContentType.objects.get_for_model(instance),
                 changed_object_id=instance.pk,
-                request_id=request.id,
-            ).update(object_data=instance.to_objectchange(action).object_data)
+                request_id=change_context.change_id,
+            )
+            m2m_changes = instance.to_objectchange(action)
+            related_changes.update(object_data=m2m_changes.object_data, object_data_v2=m2m_changes.object_data_v2)
+            objectchange = related_changes.first() if related_changes.exists() else None
         else:
             objectchange = instance.to_objectchange(action)
-            objectchange.user = _get_user_if_authenticated(request, objectchange)
-            objectchange.request_id = request.id
+            objectchange.user = _get_user_if_authenticated(change_context.get_user(), objectchange)
+            objectchange.request_id = change_context.change_id
+            objectchange.change_context = change_context.context
+            objectchange.change_context_detail = change_context.context_detail
             objectchange.save()
 
+        # Enqueue job hooks
+        if objectchange is not None:
+            enqueue_job_hooks(objectchange)
+
     # Enqueue webhooks
-    enqueue_webhooks(instance, request.user, request.id, action)
+    enqueue_webhooks(instance, change_context.get_user(), change_context.change_id, action)
 
     # Increment metric counters
     if action == ObjectChangeActionChoices.ACTION_CREATE:
@@ -90,19 +102,28 @@ def _handle_changed_object(request, sender, instance, **kwargs):
         ObjectChange.objects.filter(time__lt=cutoff).delete()
 
 
-def _handle_deleted_object(request, sender, instance, **kwargs):
+def _handle_deleted_object(change_context, sender, instance, **kwargs):
     """
     Fires when an object is deleted.
     """
+    from .jobs import enqueue_job_hooks  # avoid circular import
+
     # Record an ObjectChange if applicable
     if hasattr(instance, "to_objectchange"):
         objectchange = instance.to_objectchange(ObjectChangeActionChoices.ACTION_DELETE)
-        objectchange.user = _get_user_if_authenticated(request, objectchange)
-        objectchange.request_id = request.id
+        objectchange.user = _get_user_if_authenticated(change_context.get_user(), objectchange)
+        objectchange.request_id = change_context.change_id
+        objectchange.change_context = change_context.context
+        objectchange.change_context_detail = change_context.context_detail
         objectchange.save()
 
+        # Enqueue job hooks
+        enqueue_job_hooks(objectchange)
+
     # Enqueue webhooks
-    enqueue_webhooks(instance, request.user, request.id, ObjectChangeActionChoices.ACTION_DELETE)
+    enqueue_webhooks(
+        instance, change_context.get_user(), change_context.change_id, ObjectChangeActionChoices.ACTION_DELETE
+    )
 
     # Increment metric counters
     model_deletes.labels(instance._meta.model_name).inc()
@@ -119,6 +140,7 @@ def handle_cf_removed_obj_types(instance, action, pk_set, **kwargs):
     """
     if action == "post_remove":
         # Existing content types have been removed from the custom field, delete their data
+        # 2.0 TODO: #824 instance.slug rather than instance.name
         transaction.on_commit(lambda: delete_custom_field_data.delay(instance.name, pk_set))
 
     elif action == "post_add":
@@ -188,7 +210,7 @@ def git_repository_pre_delete(instance, **kwargs):
         job_result.set_status(JobResultStatusChoices.STATUS_COMPLETED)
     job_result.save()
 
-    # TODO: In a distributed Nautobot deployment, each Django instance and/or RQ worker instance may have its own clone
+    # TODO: In a distributed Nautobot deployment, each Django instance and/or worker instance may have its own clone
     # of this repository; we need some way to ensure that all such clones are deleted.
     # For now we just delete the one that we have locally and rely on other methods (notably get_jobs())
     # to clean up other clones as they're encountered.
@@ -196,6 +218,35 @@ def git_repository_pre_delete(instance, **kwargs):
         shutil.rmtree(instance.filesystem_path)
 
 
+#
+# Dynamic Groups
+#
+
+
+def dynamic_group_children_changed(sender, instance, action, reverse, model, pk_set, **kwargs):
+    """
+    Disallow adding DynamicGroup children if the parent has a filter.
+    """
+    if action == "pre_add" and instance.filter:
+        raise ValidationError(
+            {
+                "children": "A parent group may have either a filter or child groups, but not both. Clear the parent filter and try again."
+            }
+        )
+
+
+def dynamic_group_membership_created(sender, instance, **kwargs):
+    """
+    Forcibly call `full_clean()` when a new `DynamicGroupMembership` object
+    is manually created to prevent inadvertantly creating invalid memberships.
+    """
+    instance.full_clean()
+
+
+m2m_changed.connect(dynamic_group_children_changed, sender=DynamicGroup.children.through)
+pre_save.connect(dynamic_group_membership_created, sender=DynamicGroupMembership)
+
+
 #
 # Jobs
 #
