@@ -699,15 +699,22 @@ class JobResultTable(BaseTable):
     actions = tables.TemplateColumn(
         template_code="""
             {% load helpers %}
-            {% if record.job_model and record.job_kwargs %}
-                <a href="{% url 'extras:job_run' slug=record.job_model.slug %}?kwargs_from_job_result={{ record.pk }}"
-                   class="btn btn-xs btn-success" title="Re-run job with same arguments.">
-                    <i class="mdi mdi-repeat"></i>
-                </a>
-            {% else %}
-                <a href="#" class="btn btn-xs btn-default disabled" title="No saved job arguments, cannot be re-run">
-                    <i class="mdi mdi-repeat-off"></i>
-                </a>
+            {% if perms.extras.run_job %}
+                {% if record.job_model and record.job_kwargs %}
+                    <a href="{% url 'extras:job_run' slug=record.job_model.slug %}?kwargs_from_job_result={{ record.pk }}"
+                       class="btn btn-xs btn-success" title="Re-run job with same arguments.">
+                        <i class="mdi mdi-repeat"></i>
+                    </a>
+                {% elif record.job_model is not None %}
+                    <a href="{% url 'extras:job_run' slug=record.job_model.slug %}" class="btn btn-primary btn-xs"
+                       title="Run job">
+                        <i class="mdi mdi-play"></i>
+                    </a>
+                {% else %}
+                    <a href="#" class="btn btn-xs btn-default disabled" title="Job is not available, cannot be re-run">
+                        <i class="mdi mdi-repeat-off"></i>
+                    </a>
+                {% endif %}
             {% endif %}
             <a href="{% url 'extras:jobresult_delete' pk=record.pk %}" class="btn btn-xs btn-danger"
                title="Delete this job result.">
