@@ -2,9 +2,19 @@ import uuid
 
 from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.models import ContentType
-from django.test import TestCase
 
-from nautobot.dcim.models import Device, DeviceRole, DeviceType, Interface, Manufacturer, Platform, Rack, Region, Site
+from nautobot.dcim.filters import DeviceFilterSet
+from nautobot.dcim.models import (
+    Device,
+    DeviceRole,
+    DeviceType,
+    Interface,
+    Manufacturer,
+    Platform,
+    Rack,
+    Region,
+    Site,
+)
 from nautobot.extras.choices import (
     ObjectChangeActionChoices,
     SecretsGroupAccessTypeChoices,
@@ -12,6 +22,7 @@ from nautobot.extras.choices import (
 )
 from nautobot.extras.constants import HTTP_CONTENT_TYPE_JSON
 from nautobot.extras.filters import (
+    ComputedFieldFilterSet,
     ConfigContextFilterSet,
     CustomLinkFilterSet,
     ExportTemplateFilterSet,
@@ -19,18 +30,20 @@ from nautobot.extras.filters import (
     GraphQLQueryFilterSet,
     ImageAttachmentFilterSet,
     JobFilterSet,
+    JobHookFilterSet,
     JobLogEntryFilterSet,
     ObjectChangeFilterSet,
     RelationshipAssociationFilterSet,
     RelationshipFilterSet,
     SecretFilterSet,
-    SecretsGroupFilterSet,
     SecretsGroupAssociationFilterSet,
+    SecretsGroupFilterSet,
     StatusFilterSet,
     TagFilterSet,
     WebhookFilterSet,
 )
 from nautobot.extras.models import (
+    ComputedField,
     ConfigContext,
     CustomLink,
     ExportTemplate,
@@ -38,6 +51,7 @@ from nautobot.extras.models import (
     GraphQLQuery,
     ImageAttachment,
     Job,
+    JobHook,
     JobLogEntry,
     JobResult,
     ObjectChange,
@@ -50,17 +64,97 @@ from nautobot.extras.models import (
     Tag,
     Webhook,
 )
+from nautobot.ipam.filters import VLANFilterSet
 from nautobot.ipam.models import IPAddress, VLAN
 from nautobot.tenancy.models import Tenant, TenantGroup
 from nautobot.utilities.choices import ColorChoices
+from nautobot.utilities.testing import FilterTestCases
 from nautobot.virtualization.models import Cluster, ClusterGroup, ClusterType
 
-
 # Use the proper swappable User model
 User = get_user_model()
 
 
-class ConfigContextTestCase(TestCase):
+class ComputedFieldTestCase(FilterTestCases.FilterTestCase):
+    queryset = ComputedField.objects.all()
+    filterset = ComputedFieldFilterSet
+
+    @classmethod
+    def setUpTestData(cls):
+        ComputedField.objects.create(
+            content_type=ContentType.objects.get_for_model(Site),
+            slug="computed_field_one",
+            label="Computed Field One",
+            template="{{ obj.name }} is the name of this site.",
+            fallback_value="An error occurred while rendering this template.",
+            weight=100,
+        )
+        # Field whose template will raise a TemplateError
+        ComputedField.objects.create(
+            content_type=ContentType.objects.get_for_model(Site),
+            slug="bad_computed_field",
+            label="Bad Computed Field",
+            template="{{ something_that_throws_an_err | not_a_real_filter }} bad data",
+            fallback_value="This template has errored",
+            weight=100,
+        )
+        # Field whose template will raise a TypeError
+        ComputedField.objects.create(
+            content_type=ContentType.objects.get_for_model(Site),
+            slug="worse_computed_field",
+            label="Worse Computed Field",
+            template="{{ obj.images | list }}",
+            fallback_value="Another template error",
+            weight=200,
+        )
+        ComputedField.objects.create(
+            content_type=ContentType.objects.get_for_model(Device),
+            slug="device_computed_field",
+            label="Device Computed Field",
+            template="Hello, world.",
+            fallback_value="This template has errored",
+            weight=100,
+        )
+
+    def test_slug(self):
+        params = {"slug": ["device_computed_field", "worse_computed_field"]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_content_type(self):
+        params = {"content_type": "dcim.site"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+
+    def test_template(self):
+        params = {"template": ["Hello, world."]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+    def test_fallback_value(self):
+        params = {"fallback_value": ["This template has errored"]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_weight(self):
+        params = {"weight": [100]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+
+    def test_search(self):
+        # label
+        params = {"q": "Field One"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        # content_type__app_label
+        params = {"q": "dcim"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 4)
+        # content_type__model
+        params = {"q": "site"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        # template
+        params = {"q": "hello"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+        # fallback_value
+        params = {"q": "has errored"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+
+class ConfigContextTestCase(FilterTestCases.FilterTestCase):
     queryset = ConfigContext.objects.all()
     filterset = ConfigContextFilterSet
 
@@ -141,10 +235,6 @@ class ConfigContextTestCase(TestCase):
             c.tenant_groups.set([tenant_groups[i]])
             c.tenants.set([tenants[i]])
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_name(self):
         params = {"name": ["Config Context 1", "Config Context 2"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
@@ -222,7 +312,7 @@ class ConfigContextTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
 
-class CustomLinkTestCase(TestCase):
+class CustomLinkTestCase(FilterTestCases.FilterTestCase):
     queryset = CustomLink.objects.all()
     filterset = CustomLinkFilterSet
 
@@ -275,7 +365,7 @@ class CustomLinkTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
 
 
-class ExportTemplateTestCase(TestCase):
+class ExportTemplateTestCase(FilterTestCases.FilterTestCase):
     queryset = ExportTemplate.objects.all()
     filterset = ExportTemplateFilterSet
 
@@ -300,10 +390,6 @@ class ExportTemplateTestCase(TestCase):
             template_code="TESTING",
         )
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_name(self):
         params = {"name": ["Export Template 1", "Export Template 2"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
@@ -317,7 +403,7 @@ class ExportTemplateTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
 
 
-class GitRepositoryTestCase(TestCase):
+class GitRepositoryTestCase(FilterTestCases.FilterTestCase):
     queryset = GitRepository.objects.all()
     filterset = GitRepositoryFilterSet
 
@@ -382,7 +468,7 @@ class GitRepositoryTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
 
-class GraphQLTestCase(TestCase):
+class GraphQLTestCase(FilterTestCases.NameSlugFilterTestCase):
     queryset = GraphQLQuery.objects.all()
     filterset = GraphQLQueryFilterSet
 
@@ -498,20 +584,12 @@ query ($device: String!) {
             query.clean()
             query.save()
 
-    def test_name(self):
-        params = {"name": ["graphql-query-1"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
-
-    def test_slug(self):
-        params = {"slug": ["graphql-query-2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
-
     def test_query(self):
         params = {"query": ["sites"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
 
 
-class ImageAttachmentTestCase(TestCase):
+class ImageAttachmentTestCase(FilterTestCases.FilterTestCase):
     queryset = ImageAttachment.objects.all()
     filterset = ImageAttachmentFilterSet
 
@@ -564,10 +642,6 @@ class ImageAttachmentTestCase(TestCase):
             image_width=100,
         )
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_name(self):
         params = {"name": ["Image Attachment 1", "Image Attachment 2"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
@@ -584,31 +658,21 @@ class ImageAttachmentTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
 
-class JobFilterSetTestCase(TestCase):
+class JobFilterSetTestCase(FilterTestCases.NameSlugFilterTestCase):
     queryset = Job.objects.all()
     filterset = JobFilterSet
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_name(self):
-        params = {"name": ["File Upload Success", "File Upload Failure"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_grouping(self):
         params = {"grouping": ["test_file_upload_pass", "test_file_upload_fail"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
     def test_installed(self):
-        params = {"installed": True}
-        # 31 local jobs and 3 plugin jobs
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 34)
+        params = {"job_class_name": "TestPass", "installed": True}
+        self.assertTrue(self.filterset(params, self.queryset).qs.exists())
 
     def test_enabled(self):
-        params = {"enabled": False}
-        # 31 local jobs and 3 plugin jobs
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 34)
+        params = {"job_class_name": "TestPass", "enabled": False}
+        self.assertTrue(self.filterset(params, self.queryset).qs.exists())
 
     def test_commit_default(self):
         params = {"commit_default": False}
@@ -633,8 +697,85 @@ class JobFilterSetTestCase(TestCase):
         params = {"q": value}
         self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
+    def test_is_job_hook_receiver(self):
+        params = {"is_job_hook_receiver": True}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 4)
 
-class JobLogEntryTestCase(TestCase):
+
+class JobHookFilterSetTestCase(FilterTestCases.NameSlugFilterTestCase):
+    queryset = JobHook.objects.all()
+    filterset = JobHookFilterSet
+
+    @classmethod
+    def setUpTestData(cls):
+        job_hooks = (
+            JobHook.objects.create(
+                name="JobHook1",
+                job=Job.objects.get(job_class_name="TestJobHookReceiverLog"),
+                type_create=True,
+                type_update=True,
+                type_delete=True,
+            ),
+            JobHook.objects.create(
+                name="JobHook2",
+                job=Job.objects.get(job_class_name="TestJobHookReceiverChange"),
+                type_create=True,
+                type_update=True,
+                type_delete=False,
+            ),
+            JobHook.objects.create(
+                name="JobHook3",
+                enabled=False,
+                job=Job.objects.get(job_class_name="TestJobHookReceiverFail"),
+                type_delete=True,
+            ),
+        )
+
+        devicetype_ct = ContentType.objects.get_for_model(DeviceType)
+        job_hooks[0].content_types.set([devicetype_ct])
+        job_hooks[1].content_types.set([devicetype_ct])
+
+    def test_name(self):
+        params = {"name": ["JobHook1", "JobHook2"]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_content_types(self):
+        params = {"content_types": ["dcim.devicetype"]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_enabled(self):
+        params = {"enabled": True}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_job(self):
+        jobs = Job.objects.filter(job_class_name__in=["TestJobHookReceiverLog", "TestJobHookReceiverChange"])[:2]
+        params = {"job": [jobs[0].slug, jobs[1].pk]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_slug(self):
+        params = {"slug": ["jobhook1", "jobhook2"]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_type_create(self):
+        params = {"type_create": True}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_type_delete(self):
+        params = {"type_delete": True}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_type_update(self):
+        params = {"type_update": True}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+
+    def test_search(self):
+        params = {"q": "hook"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+        params = {"q": "hook1"}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
+
+
+class JobLogEntryTestCase(FilterTestCases.FilterTestCase):
     queryset = JobLogEntry.objects.all()
     filterset = JobLogEntryFilterSet
 
@@ -654,10 +795,6 @@ class JobLogEntryTestCase(TestCase):
                 message=f"I am a {log_level} log.",
             )
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_log_level(self):
         params = {"log_level": "success"}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
@@ -679,7 +816,7 @@ class JobLogEntryTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
 
-class ObjectChangeTestCase(TestCase):
+class ObjectChangeTestCase(FilterTestCases.FilterTestCase):
     queryset = ObjectChange.objects.all()
     filterset = ObjectChangeFilterSet
 
@@ -749,10 +886,6 @@ class ObjectChangeTestCase(TestCase):
             object_data={"address": str(ipaddress.address), "status": ipaddress.status},
         )
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:3]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
-
     def test_user(self):
         params = {"user_id": User.objects.filter(username__in=["user1", "user2"]).values_list("pk", flat=True)}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 4)
@@ -777,7 +910,7 @@ class ObjectChangeTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
 
-class RelationshipTestCase(TestCase):
+class RelationshipTestCase(FilterTestCases.NameSlugFilterTestCase):
     queryset = Relationship.objects.all()
     filterset = RelationshipFilterSet
 
@@ -809,14 +942,6 @@ class RelationshipTestCase(TestCase):
             destination_type=interface_type,
         ).validated_save()
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_name(self):
-        params = {"name": ["Primary VLAN", "Primary Interface"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_type(self):
         params = {"type": "one-to-many"}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
@@ -830,7 +955,7 @@ class RelationshipTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
 
-class RelationshipAssociationTestCase(TestCase):
+class RelationshipAssociationTestCase(FilterTestCases.FilterTestCase):
     queryset = RelationshipAssociation.objects.all()
     filterset = RelationshipAssociationFilterSet
 
@@ -854,6 +979,13 @@ class RelationshipAssociationTestCase(TestCase):
                 source_type=cls.vlan_type,
                 destination_type=cls.device_type,
             ),
+            Relationship(
+                name="Device Device",
+                slug="symmetric-device-device",
+                type="symmetric-many-to-many",
+                source_type=cls.device_type,
+                destination_type=cls.device_type,
+            ),
         )
         for relationship in cls.relationships:
             relationship.validated_save()
@@ -865,6 +997,7 @@ class RelationshipAssociationTestCase(TestCase):
         cls.devices = (
             Device.objects.create(name="Device 1", device_type=devicetype, device_role=devicerole, site=site),
             Device.objects.create(name="Device 2", device_type=devicetype, device_role=devicerole, site=site),
+            Device.objects.create(name="Device 3", device_type=devicetype, device_role=devicerole, site=site),
         )
         cls.vlans = (
             VLAN.objects.create(vid=1, name="VLAN 1"),
@@ -899,10 +1032,20 @@ class RelationshipAssociationTestCase(TestCase):
             destination_type=cls.device_type,
             destination_id=cls.devices[1].pk,
         ).validated_save()
-
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        RelationshipAssociation(
+            relationship=cls.relationships[2],
+            source_type=cls.device_type,
+            source_id=cls.devices[0].pk,
+            destination_type=cls.device_type,
+            destination_id=cls.devices[1].pk,
+        ).validated_save()
+        RelationshipAssociation(
+            relationship=cls.relationships[2],
+            source_type=cls.device_type,
+            source_id=cls.devices[1].pk,
+            destination_type=cls.device_type,
+            destination_id=cls.devices[2].pk,
+        ).validated_save()
 
     def test_relationship(self):
         params = {"relationship": [self.relationships[0].slug]}
@@ -910,22 +1053,215 @@ class RelationshipAssociationTestCase(TestCase):
 
     def test_source_type(self):
         params = {"source_type": ["dcim.device", "dcim.interface"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 4)
 
     def test_source_id(self):
         params = {"source_id": [self.devices[0].pk, self.devices[1].pk]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 4)
 
     def test_destination_type(self):
         params = {"destination_type": ["dcim.device", "dcim.interface"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 4)
 
     def test_destination_id(self):
         params = {"destination_id": [self.devices[0].pk, self.devices[1].pk]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 3)
+
+    def test_peer_id(self):
+        params = {"peer_id": [self.devices[0].pk, self.devices[1].pk]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
+        params = {"peer_id": [self.devices[2].pk]}
+        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 1)
 
 
-class SecretTestCase(TestCase):
+class RelationshipModelFilterSetTestCase(FilterTestCases.FilterTestCase):
+    queryset = RelationshipAssociation.objects.all()
+    filterset = RelationshipAssociationFilterSet
+
+    @classmethod
+    def setUpTestData(cls):
+        cls.device_type = ContentType.objects.get_for_model(Device)
+        cls.vlan_type = ContentType.objects.get_for_model(VLAN)
+        cls.relationships = (
+            Relationship(
+                name="Device VLANs",
+                slug="device-vlans",
+                type="many-to-many",
+                source_type=cls.device_type,
+                destination_type=cls.vlan_type,
+            ),
+            Relationship(
+                name="Primary VLAN",
+                slug="primary-vlan",
+                type="one-to-many",
+                source_type=cls.vlan_type,
+                destination_type=cls.device_type,
+            ),
+            Relationship(
+                name="Device Peers",
+                slug="device-peers",
+                type="symmetric-many-to-many",
+                source_type=cls.device_type,
+                destination_type=cls.device_type,
+            ),
+        )
+        for relationship in cls.relationships:
+            relationship.validated_save()
+
+        manufacturer = Manufacturer.objects.create(name="Manufacturer 1", slug="manufacturer-1")
+        devicetype = DeviceType.objects.create(manufacturer=manufacturer, model="Device Type 1", slug="device-type-1")
+        devicerole = DeviceRole.objects.create(name="Device Role 1", slug="device-role-1")
+        site = Site.objects.create(name="Site 1", slug="site-1")
+        cls.devices = (
+            Device.objects.create(name="Device 1", device_type=devicetype, device_role=devicerole, site=site),
+            Device.objects.create(name="Device 2", device_type=devicetype, device_role=devicerole, site=site),
+            Device.objects.create(name="Device 3", device_type=devicetype, device_role=devicerole, site=site),
+        )
+        cls.vlans = (
+            VLAN.objects.create(vid=1, name="VLAN 1"),
+            VLAN.objects.create(vid=2, name="VLAN 2"),
+            VLAN.objects.create(vid=3, name="VLAN 3"),
+        )
+        cls.relationship_associations = (
+            RelationshipAssociation(
+                relationship=cls.relationships[0],
+                source_type=cls.device_type,
+                source_id=cls.devices[0].pk,
+                destination_type=cls.vlan_type,
+                destination_id=cls.vlans[0].pk,
+            ),
+            RelationshipAssociation(
+                relationship=cls.relationships[0],
+                source_type=cls.device_type,
+                source_id=cls.devices[1].pk,
+                destination_type=cls.vlan_type,
+                destination_id=cls.vlans[1].pk,
+            ),
+            RelationshipAssociation(
+                relationship=cls.relationships[1],
+                source_type=cls.vlan_type,
+                source_id=cls.vlans[0].pk,
+                destination_type=cls.device_type,
+                destination_id=cls.devices[0].pk,
+            ),
+            RelationshipAssociation(
+                relationship=cls.relationships[1],
+                source_type=cls.vlan_type,
+                source_id=cls.vlans[0].pk,
+                destination_type=cls.device_type,
+                destination_id=cls.devices[1].pk,
+            ),
+            RelationshipAssociation(
+                relationship=cls.relationships[1],
+                source_type=cls.vlan_type,
+                source_id=cls.vlans[0].pk,
+                destination_type=cls.device_type,
+                destination_id=cls.devices[2].pk,
+            ),
+            RelationshipAssociation(
+                relationship=cls.relationships[2],
+                source_type=cls.device_type,
+                source_id=cls.devices[0].pk,
+                destination_type=cls.device_type,
+                destination_id=cls.devices[1].pk,
+            ),
+            RelationshipAssociation(
+                relationship=cls.relationships[2],
+                source_type=cls.device_type,
+                source_id=cls.devices[0].pk,
+                destination_type=cls.device_type,
+                destination_id=cls.devices[2].pk,
+            ),
+            RelationshipAssociation(
+                relationship=cls.relationships[2],
+                source_type=cls.device_type,
+                source_id=cls.devices[1].pk,
+                destination_type=cls.device_type,
+                destination_id=cls.devices[2].pk,
+            ),
+        )
+        for relationship_association in cls.relationship_associations:
+            relationship_association.validated_save()
+
+    def test_one_to_many_source(self):
+        self.queryset = Device.objects.all()
+        self.filterset = DeviceFilterSet
+        self.assertEqual(
+            self.filterset({f"cr_{self.relationships[1].slug}__source": [self.vlans[0].pk]}, self.queryset).qs.count(),
+            3,
+        )
+
+    def test_one_to_many_destination(self):
+        self.queryset = VLAN.objects.all()
+        self.filterset = VLANFilterSet
+        self.assertEqual(
+            self.filterset(
+                {f"cr_{self.relationships[1].slug}__destination": [self.devices[0].pk, self.devices[1].pk]},
+                self.queryset,
+            ).qs.count(),
+            1,
+        )
+
+    def test_many_to_many_source(self):
+        self.queryset = VLAN.objects.all()
+        self.filterset = VLANFilterSet
+        self.assertEqual(
+            self.filterset(
+                {f"cr_{self.relationships[0].slug}__source": [self.devices[0].pk, self.devices[1].pk]}, self.queryset
+            ).qs.count(),
+            2,
+        )
+
+    def test_many_to_many_destination(self):
+        self.queryset = Device.objects.all()
+        self.filterset = DeviceFilterSet
+        self.assertEqual(
+            self.filterset(
+                {f"cr_{self.relationships[0].slug}__destination": [self.vlans[0].pk, self.vlans[1].pk]}, self.queryset
+            ).qs.count(),
+            2,
+        )
+
+    def test_many_to_many_peer(self):
+        self.queryset = Device.objects.all()
+        self.filterset = DeviceFilterSet
+        self.assertEqual(
+            self.filterset(
+                {f"cr_{self.relationships[2].slug}__peer": [self.devices[0].pk, self.devices[1].pk]}, self.queryset
+            ).qs.count(),
+            3,
+        )
+        self.assertEqual(
+            self.filterset({f"cr_{self.relationships[2].slug}__peer": [self.devices[2].pk]}, self.queryset).qs.count(),
+            2,
+        )
+
+    def test_combination(self):
+        self.queryset = Device.objects.all()
+        self.filterset = DeviceFilterSet
+        self.assertEqual(
+            self.filterset(
+                {
+                    f"cr_{self.relationships[2].slug}__peer": [self.devices[0].pk, self.devices[1].pk],
+                    f"cr_{self.relationships[0].slug}__destination": [self.vlans[0].pk, self.vlans[1].pk],
+                },
+                self.queryset,
+            ).qs.count(),
+            2,
+        )
+        self.assertEqual(
+            self.filterset(
+                {
+                    f"cr_{self.relationships[2].slug}__peer": [self.devices[2].pk],
+                    f"cr_{self.relationships[0].slug}__destination": [self.vlans[0].pk, self.vlans[1].pk],
+                },
+                self.queryset,
+            ).qs.count(),
+            2,
+        )
+
+
+class SecretTestCase(FilterTestCases.NameSlugFilterTestCase):
     queryset = Secret.objects.all()
     filterset = SecretFilterSet
 
@@ -952,18 +1288,6 @@ class SecretTestCase(TestCase):
         for secret in secrets:
             secret.validated_save()
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_name(self):
-        params = {"name": ["Secret 1", "Secret 2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_slug(self):
-        params = {"slug": ["secret-1", "secret-2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_provider(self):
         params = {"provider": ["environment-variable"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
@@ -974,7 +1298,7 @@ class SecretTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
 
-class SecretsGroupTestCase(TestCase):
+class SecretsGroupTestCase(FilterTestCases.NameSlugFilterTestCase):
     queryset = SecretsGroup.objects.all()
     filterset = SecretsGroupFilterSet
 
@@ -984,25 +1308,13 @@ class SecretsGroupTestCase(TestCase):
         SecretsGroup.objects.create(name="Group 2", slug="group-2")
         SecretsGroup.objects.create(name="Group 3", slug="group-3")
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_name(self):
-        params = {"name": ["Group 1", "Group 2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_slug(self):
-        params = {"slug": ["group-1", "group-2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_search(self):
         value = self.queryset.values_list("pk", flat=True)[0]
         params = {"q": value}
         self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
 
-class SecretsGroupAssociationTestCase(TestCase):
+class SecretsGroupAssociationTestCase(FilterTestCases.FilterTestCase):
     queryset = SecretsGroupAssociation.objects.all()
     filterset = SecretsGroupAssociationFilterSet
 
@@ -1054,10 +1366,6 @@ class SecretsGroupAssociationTestCase(TestCase):
             secret_type=SecretsGroupSecretTypeChoices.TYPE_PASSWORD,
         )
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_group(self):
         params = {"group_id": [self.groups[0].pk, self.groups[1].pk]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
@@ -1079,7 +1387,7 @@ class SecretsGroupAssociationTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
 
 
-class StatusTestCase(TestCase):
+class StatusTestCase(FilterTestCases.NameSlugFilterTestCase):
     queryset = Status.objects.all()
     filterset = StatusFilterSet
 
@@ -1096,18 +1404,6 @@ class StatusTestCase(TestCase):
         See `extras.management.create_custom_statuses` for context.
         """
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_name(self):
-        params = {"name": ["Active", "Offline"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_slug(self):
-        params = {"slug": ["active", "offline"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_content_types(self):
         ct = ContentType.objects.get_for_model(Device)
         status_count = self.queryset.filter(content_types=ct).count()
@@ -1121,7 +1417,7 @@ class StatusTestCase(TestCase):
         # imported by way of `extras.management.create_custom_statuses`. If as
         # these objects are imported, and this test fails, this number will need
         # to be adjusted.
-        expected_count = 3
+        expected_count = 4
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), expected_count)
 
     def test_search(self):
@@ -1132,7 +1428,7 @@ class StatusTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
 
-class TagTestCase(TestCase):
+class TagTestCase(FilterTestCases.NameSlugFilterTestCase):
     queryset = Tag.objects.all()
     filterset = TagFilterSet
 
@@ -1149,18 +1445,6 @@ class TagTestCase(TestCase):
         for tag in cls.tags[1:]:
             tag.content_types.add(ContentType.objects.get_for_model(Device))
 
-    def test_id(self):
-        params = {"id": self.queryset.values_list("pk", flat=True)[:2]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_name(self):
-        params = {"name": ["Tag 1", "Tag 2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
-    def test_slug(self):
-        params = {"slug": ["tag-1", "tag-2"]}
-        self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
-
     def test_color(self):
         params = {"color": ["ff0000", "00ff00"]}
         self.assertEqual(self.filterset(params, self.queryset).qs.count(), 2)
@@ -1179,7 +1463,7 @@ class TagTestCase(TestCase):
         self.assertEqual(self.filterset(params, self.queryset).qs.values_list("pk", flat=True)[0], value)
 
 
-class WebhookTestCase(TestCase):
+class WebhookTestCase(FilterTestCases.FilterTestCase):
     queryset = Webhook.objects.all()
     filterset = WebhookFilterSet
 
