@@ -4,6 +4,7 @@ import uuid
 
 from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.models import ContentType
+from django.core.exceptions import ValidationError
 from django.test import override_settings
 from django.urls import reverse
 from django.utils import timezone
@@ -30,6 +31,7 @@ from nautobot.extras.models import (
     GraphQLQuery,
     Job,
     JobResult,
+    Note,
     ObjectChange,
     Relationship,
     RelationshipAssociation,
@@ -47,6 +49,7 @@ from nautobot.ipam.models import VLAN, VLANGroup
 from nautobot.users.models import ObjectPermission
 from nautobot.utilities.testing import ViewTestCases, TestCase, extract_page_body, extract_form_failures
 from nautobot.utilities.testing.utils import disable_warnings, post_data
+from nautobot.utilities.utils import slugify_dashes_to_underscores
 
 
 # Use the proper swappable User model
@@ -63,6 +66,8 @@ class ComputedFieldTestCase(
     ViewTestCases.ListObjectsViewTestCase,
 ):
     model = ComputedField
+    slug_source = "label"
+    slugify_function = staticmethod(slugify_dashes_to_underscores)
 
     @classmethod
     def setUpTestData(cls):
@@ -116,7 +121,6 @@ class ComputedFieldTestCase(
             "weight": 100,
         }
 
-        cls.slug_source = "label"
         cls.slug_test_object = "Computed Field Five"
 
 
@@ -346,15 +350,18 @@ class CustomLinkTestCase(
 
 
 class CustomFieldTestCase(
+    # No NotesViewTestCase or BulkImportObjectsViewTestCase, at least for now
     ViewTestCases.BulkDeleteObjectsViewTestCase,
     ViewTestCases.CreateObjectViewTestCase,
     ViewTestCases.DeleteObjectViewTestCase,
     ViewTestCases.EditObjectViewTestCase,
     ViewTestCases.GetObjectViewTestCase,
+    ViewTestCases.GetObjectChangelogViewTestCase,
     ViewTestCases.ListObjectsViewTestCase,
 ):
     model = CustomField
-    reverse_url_attribute = "name"
+    slug_source = "label"
+    slugify_function = staticmethod(slugify_dashes_to_underscores)
 
     @classmethod
     def setUpTestData(cls):
@@ -379,16 +386,24 @@ class CustomFieldTestCase(
                 label="Custom Field Integer",
                 default="",
             ),
+            CustomField(
+                type=CustomFieldTypeChoices.TYPE_TEXT,
+                # https://github.com/nautobot/nautobot/issues/1962
+                name="Custom field? With special / unusual characters!",
+                default="",
+            ),
         ]
 
+        cls.slug_test_object = "Custom Field Integer"
+
         for custom_field in custom_fields:
             custom_field.validated_save()
             custom_field.content_types.set([obj_type])
 
         cls.form_data = {
             "content_types": [obj_type.pk],
-            "type": CustomFieldTypeChoices.TYPE_BOOLEAN,
-            "name": "Custom Field Boolean",
+            "type": CustomFieldTypeChoices.TYPE_BOOLEAN,  # type is mandatory but cannot be changed once set.
+            "slug": "custom_field_boolean",  # slug is mandatory but cannot be changed once set.
             "label": "Custom Field Boolean",
             "default": None,
             "filter_logic": "loose",
@@ -401,21 +416,25 @@ class CustomFieldTestCase(
         }
 
     def test_create_object_without_permission(self):
-        # Can't have two CustomFields with the same "name"
-        for cf in CustomField.objects.all():
-            cf.delete()
+        # Can't have two CustomFields with the same "slug"
+        self.form_data = self.form_data.copy()
+        self.form_data["slug"] = "custom_field_boolean_2"
         super().test_create_object_without_permission()
 
     def test_create_object_with_permission(self):
-        # Can't have two CustomFields with the same "name"
-        for cf in CustomField.objects.all():
-            cf.delete()
+        # Can't have two CustomFields with the same "slug"
+        self.form_data = self.form_data.copy()
+        self.form_data["slug"] = "custom_field_boolean_2"
         super().test_create_object_with_permission()
+        instance = self._get_queryset().get(slug="custom_field_boolean_2")
+        # 2.0 TODO: #824 removal of `name` field altogether
+        # Assure that `name` was auto-populated from the given slug
+        self.assertEqual(instance.name, instance.slug)
 
     def test_create_object_with_constrained_permission(self):
-        # Can't have two CustomFields with the same "name"
-        for cf in CustomField.objects.all():
-            cf.delete()
+        # Can't have two CustomFields with the same "slug"
+        self.form_data = self.form_data.copy()
+        self.form_data["slug"] = "custom_field_boolean_2"
         super().test_create_object_with_constrained_permission()
 
 
@@ -470,6 +489,11 @@ class DynamicGroupTestCase(
             "slug": "new-dynamic-group",
             "description": "I am a new dynamic group object.",
             "content_type": content_type.pk,
+            # Management form fields required for the dynamic formset
+            "dynamic_group_memberships-TOTAL_FORMS": "0",
+            "dynamic_group_memberships-INITIAL_FORMS": "1",
+            "dynamic_group_memberships-MIN_NUM_FORMS": "0",
+            "dynamic_group_memberships-MAX_NUM_FORMS": "1000",
         }
 
 
@@ -569,6 +593,48 @@ class GitRepositoryTestCase(
         cls.slug_test_object = "Repo 4"
 
 
+class NoteTestCase(
+    ViewTestCases.CreateObjectViewTestCase,
+    ViewTestCases.DeleteObjectViewTestCase,
+    ViewTestCases.EditObjectViewTestCase,
+    ViewTestCases.GetObjectChangelogViewTestCase,
+):
+    model = Note
+
+    @classmethod
+    def setUpTestData(cls):
+
+        content_type = ContentType.objects.get_for_model(Site)
+        site = Site.objects.create(name="Site 1", slug="site-1")
+        user = User.objects.first()
+
+        # Notes Objects to test
+        Note.objects.create(
+            note="Site has been placed on maintenance.",
+            user=user,
+            assigned_object_type=content_type,
+            assigned_object_id=site.pk,
+        ),
+        Note.objects.create(
+            note="Site maintenance has ended.",
+            user=user,
+            assigned_object_type=content_type,
+            assigned_object_id=site.pk,
+        ),
+        Note.objects.create(
+            note="Site is under duress.",
+            user=user,
+            assigned_object_type=content_type,
+            assigned_object_id=site.pk,
+        ),
+
+        cls.form_data = {
+            "note": "This is Site note.",
+            "assigned_object_type": content_type.pk,
+            "assigned_object_id": site.pk,
+        }
+
+
 # Not a full-fledged PrimaryObjectViewTestCase as there's no BulkEditView for Secrets
 class SecretTestCase(
     ViewTestCases.GetObjectViewTestCase,
@@ -881,6 +947,57 @@ class ScheduledJobTestCase(
         self.assertHttpStatus(response, 200)
         self.assertNotIn("test4", extract_page_body(response.content.decode(response.charset)))
 
+    def test_non_valid_crontab_syntax(self):
+        self.add_permissions("extras.view_scheduledjob")
+
+        def scheduled_job_factory(name, crontab):
+            ScheduledJob.objects.create(
+                enabled=True,
+                name=name,
+                task="nautobot.extras.jobs.scheduled_job_handler",
+                job_class="local/test_pass/TestPass",
+                interval=JobExecutionType.TYPE_CUSTOM,
+                user=self.user,
+                start_time=timezone.now(),
+                crontab=crontab,
+            )
+
+        with self.assertRaises(ValidationError):
+            scheduled_job_factory("test5", None)
+
+        with self.assertRaises(ValidationError):
+            scheduled_job_factory("test6", "")
+
+        with self.assertRaises(ValidationError):
+            scheduled_job_factory("test7", "not_enough_values_to_unpack")
+
+        with self.assertRaises(ValidationError):
+            scheduled_job_factory("test8", "one too many values to unpack")
+
+        with self.assertRaises(ValidationError):
+            scheduled_job_factory("test9", "-1 * * * *")
+
+        with self.assertRaises(ValidationError):
+            scheduled_job_factory("test10", "invalid literal * * *")
+
+    def test_valid_crontab_syntax(self):
+        self.add_permissions("extras.view_scheduledjob")
+
+        ScheduledJob.objects.create(
+            enabled=True,
+            name="test11",
+            task="nautobot.extras.jobs.scheduled_job_handler",
+            job_class="local/test_pass/TestPass",
+            interval=JobExecutionType.TYPE_CUSTOM,
+            user=self.user,
+            start_time=datetime.now(),
+            crontab="*/15 9,17 3 * 1-5",
+        )
+
+        response = self.client.get(self._get_url("list"))
+        self.assertHttpStatus(response, 200)
+        self.assertIn("test11", extract_page_body(response.content.decode(response.charset)))
+
 
 class ApprovalQueueTestCase(
     # It would be nice to use ViewTestCases.GetObjectViewTestCase as well,
@@ -1309,8 +1426,8 @@ class JobTestCase(
     model = Job
 
     def _get_queryset(self):
-        """Don't include hidden or non-installed Jobs as they won't appear in the UI by default."""
-        return self.model.objects.filter(installed=True, hidden=False)
+        """Don't include hidden Jobs, non-installed Jobs or JobHookReceivers as they won't appear in the UI by default."""
+        return self.model.objects.filter(installed=True, hidden=False, is_job_hook_receiver=False)
 
     @classmethod
     def setUpTestData(cls):
@@ -1376,6 +1493,8 @@ class JobTestCase(
             "soft_time_limit": 350,
             "time_limit_override": True,
             "time_limit": 650,
+            "has_sensitive_variables": False,
+            "has_sensitive_variables_override": True,
         }
 
     #
@@ -1638,6 +1757,28 @@ class JobTestCase(
             scheduled = ScheduledJob.objects.last()
             self.assertEqual(scheduled.kwargs["scheduled_job_pk"], str(scheduled.pk))
 
+    @mock.patch("nautobot.extras.views.get_worker_count", return_value=1)
+    def test_run_job_with_sensitive_variables_for_future(self, _):
+        self.add_permissions("extras.run_job")
+        self.add_permissions("extras.view_scheduledjob")
+
+        self.test_pass.has_sensitive_variables = True
+        self.test_pass.has_sensitive_variables_override = True
+        self.test_pass.validated_save()
+
+        start_time = timezone.now() + timedelta(minutes=1)
+        data = {
+            "_schedule_type": "future",
+            "_schedule_name": "test",
+            "_schedule_start_time": str(start_time),
+        }
+        for run_url in self.run_urls:
+            response = self.client.post(run_url, data)
+            self.assertHttpStatus(response, 200, msg=self.run_urls[1])
+
+            content = extract_page_body(response.content.decode(response.charset))
+            self.assertIn("Unable to schedule job: Job may have sensitive input variables.", content)
+
 
 # TODO: Convert to StandardTestCases.Views
 class ObjectChangeTestCase(TestCase):
@@ -1679,11 +1820,13 @@ class RelationshipTestCase(
     ViewTestCases.DeleteObjectViewTestCase,
     ViewTestCases.EditObjectViewTestCase,
     ViewTestCases.BulkDeleteObjectsViewTestCase,
-    # TODO? ViewTestCases.GetObjectViewTestCase,
-    # TODO? ViewTestCases.GetObjectChangelogViewTestCase,
+    ViewTestCases.GetObjectViewTestCase,
+    ViewTestCases.GetObjectChangelogViewTestCase,
     ViewTestCases.ListObjectsViewTestCase,
 ):
     model = Relationship
+    slug_source = "name"
+    slugify_function = staticmethod(slugify_dashes_to_underscores)
 
     @classmethod
     def setUpTestData(cls):
@@ -1726,7 +1869,6 @@ class RelationshipTestCase(
             "destination_filter": None,
         }
 
-        cls.slug_source = "name"
         cls.slug_test_object = "Primary Interface"
 
 
