@@ -7,7 +7,7 @@ from rest_framework.exceptions import APIException
 from rest_framework.response import Response
 from rest_framework.routers import APIRootView
 
-from nautobot.extras.api.views import CustomFieldModelViewSet, StatusViewSetMixin
+from nautobot.extras.api.views import NautobotModelViewSet, StatusViewSetMixin
 from nautobot.ipam import filters
 from nautobot.ipam.models import (
     Aggregate,
@@ -22,7 +22,11 @@ from nautobot.ipam.models import (
     VRF,
 )
 from nautobot.utilities.config import get_settings_or_config
-from nautobot.utilities.utils import count_related
+from nautobot.utilities.utils import (
+    count_related,
+    SerializerForAPIVersions,
+    versioned_serializer_selector,
+)
 from . import serializers
 
 
@@ -40,7 +44,7 @@ class IPAMRootView(APIRootView):
 #
 
 
-class VRFViewSet(CustomFieldModelViewSet):
+class VRFViewSet(NautobotModelViewSet):
     queryset = (
         VRF.objects.prefetch_related("tenant")
         .prefetch_related("import_targets", "export_targets", "tags")
@@ -58,7 +62,7 @@ class VRFViewSet(CustomFieldModelViewSet):
 #
 
 
-class RouteTargetViewSet(CustomFieldModelViewSet):
+class RouteTargetViewSet(NautobotModelViewSet):
     queryset = RouteTarget.objects.prefetch_related("tenant").prefetch_related("tags")
     serializer_class = serializers.RouteTargetSerializer
     filterset_class = filters.RouteTargetFilterSet
@@ -69,7 +73,7 @@ class RouteTargetViewSet(CustomFieldModelViewSet):
 #
 
 
-class RIRViewSet(CustomFieldModelViewSet):
+class RIRViewSet(NautobotModelViewSet):
     queryset = RIR.objects.annotate(aggregate_count=count_related(Aggregate, "rir"))
     serializer_class = serializers.RIRSerializer
     filterset_class = filters.RIRFilterSet
@@ -80,7 +84,7 @@ class RIRViewSet(CustomFieldModelViewSet):
 #
 
 
-class AggregateViewSet(CustomFieldModelViewSet):
+class AggregateViewSet(NautobotModelViewSet):
     queryset = Aggregate.objects.prefetch_related("rir").prefetch_related("tags")
     serializer_class = serializers.AggregateSerializer
     filterset_class = filters.AggregateFilterSet
@@ -91,7 +95,7 @@ class AggregateViewSet(CustomFieldModelViewSet):
 #
 
 
-class RoleViewSet(CustomFieldModelViewSet):
+class RoleViewSet(NautobotModelViewSet):
     queryset = Role.objects.annotate(
         prefix_count=count_related(Prefix, "role"),
         vlan_count=count_related(VLAN, "role"),
@@ -105,7 +109,7 @@ class RoleViewSet(CustomFieldModelViewSet):
 #
 
 
-class PrefixViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
+class PrefixViewSet(StatusViewSetMixin, NautobotModelViewSet):
     queryset = Prefix.objects.prefetch_related(
         "role",
         "site",
@@ -303,7 +307,7 @@ class PrefixViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
     retrieve=extend_schema(responses={"200": serializers.IPAddressSerializerLegacy}, versions=["1.2"]),
     update=extend_schema(responses={"200": serializers.IPAddressSerializerLegacy}, versions=["1.2"]),
 )
-class IPAddressViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
+class IPAddressViewSet(StatusViewSetMixin, NautobotModelViewSet):
     queryset = IPAddress.objects.prefetch_related(
         "assigned_object",
         "nat_inside",
@@ -316,6 +320,16 @@ class IPAddressViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
     serializer_class = serializers.IPAddressSerializer
     filterset_class = filters.IPAddressFilterSet
 
+    def get_serializer_class(self):
+        serializer_choices = (
+            SerializerForAPIVersions(versions=["1.2"], serializer=serializers.IPAddressSerializerLegacy),
+        )
+        return versioned_serializer_selector(
+            obj=self,
+            serializer_choices=serializer_choices,
+            default_serializer=super().get_serializer_class(),
+        )
+
     # 2.0 TODO: Remove exception class and overloaded methods below
     # Because serializer has nat_outside as read_only, update and create methods do not need to be overloaded
     class NATOutsideIncompatibleLegacyBehavior(APIException):
@@ -323,21 +337,6 @@ class IPAddressViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
         default_detail = "This object does not conform to pre-1.3 behavior. Please correct data or use API version 1.3"
         default_code = "precondition_failed"
 
-    def get_serializer_class(self):
-        if (
-            not self.brief
-            and not getattr(self, "swagger_fake_view", False)
-            and (
-                not hasattr(self.request, "major_version")
-                or self.request.major_version > 1
-                or (self.request.major_version == 1 and self.request.minor_version < 3)
-            )
-        ):
-            # API version 1.2 or earlier - use the legacy serializer
-            # Note: Generating API docs at this point request doesn't define major_version or minor_version for some reason
-            return serializers.IPAddressSerializerLegacy
-        return super().get_serializer_class()
-
     def retrieve(self, request, pk=None):
         try:
             return super().retrieve(request, pk)
@@ -358,7 +357,7 @@ class IPAddressViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
 #
 
 
-class VLANGroupViewSet(CustomFieldModelViewSet):
+class VLANGroupViewSet(NautobotModelViewSet):
     queryset = VLANGroup.objects.prefetch_related("site").annotate(vlan_count=count_related(VLAN, "group"))
     serializer_class = serializers.VLANGroupSerializer
     filterset_class = filters.VLANGroupFilterSet
@@ -369,7 +368,7 @@ class VLANGroupViewSet(CustomFieldModelViewSet):
 #
 
 
-class VLANViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
+class VLANViewSet(StatusViewSetMixin, NautobotModelViewSet):
     queryset = VLAN.objects.prefetch_related(
         "group",
         "site",
@@ -387,7 +386,7 @@ class VLANViewSet(StatusViewSetMixin, CustomFieldModelViewSet):
 #
 
 
-class ServiceViewSet(CustomFieldModelViewSet):
+class ServiceViewSet(NautobotModelViewSet):
     queryset = Service.objects.prefetch_related("device", "virtual_machine", "tags", "ipaddresses")
     serializer_class = serializers.ServiceSerializer
     filterset_class = filters.ServiceFilterSet
