@@ -4,7 +4,7 @@ from django_tables2.utils import Accessor
 
 from nautobot.dcim.models import Interface
 from nautobot.extras.tables import StatusTableMixin
-from nautobot.tenancy.tables import COL_TENANT
+from nautobot.tenancy.tables import TenantColumn
 from nautobot.utilities.tables import (
     BaseTable,
     BooleanColumn,
@@ -14,6 +14,7 @@ from nautobot.utilities.tables import (
     TagColumn,
     ToggleColumn,
 )
+from nautobot.utilities.templatetags.helpers import render_boolean
 from nautobot.virtualization.models import VMInterface
 from .models import (
     Aggregate,
@@ -87,7 +88,7 @@ VLAN_LINK = """
 {% if record.present_in_database %}
     <a href="{{ record.get_absolute_url }}">{{ record.vid }}</a>
 {% elif perms.ipam.add_vlan %}
-    <a href="{% url 'ipam:vlan_add' %}?vid={{ record.vid }}&group={{ vlan_group.pk }}{% if vlan_group.site %}&site={{ vlan_group.site.pk }}{% endif %}" class="btn btn-xs btn-success">{{ record.available }} VLAN{{ record.available|pluralize }} available</a>
+    <a href="{% url 'ipam:vlan_add' %}?vid={{ record.vid }}&group={{ vlan_group.pk }}{% if vlan_group.site %}&site={{ vlan_group.site.pk }}{% endif %}{% if vlan_group.location %}&location={{ vlan_group.location.pk }}{% endif %}" class="btn btn-xs btn-success">{{ record.available }} VLAN{{ record.available|pluralize }} available</a>
 {% else %}
     {{ record.available }} VLAN{{ record.available|pluralize }} available
 {% endif %}
@@ -112,34 +113,13 @@ VLAN_ROLE_LINK = """
 VLANGROUP_ADD_VLAN = """
 {% with next_vid=record.get_next_available_vid %}
     {% if next_vid and perms.ipam.add_vlan %}
-        <a href="{% url 'ipam:vlan_add' %}?site={{ record.site_id }}&group={{ record.pk }}&vid={{ next_vid }}" title="Add VLAN" class="btn btn-xs btn-success">
+        <a href="{% url 'ipam:vlan_add' %}?site={{ record.site_id }}{% if record.location %}&location={{ record.location_id }}{% endif %}&group={{ record.pk }}&vid={{ next_vid }}" title="Add VLAN" class="btn btn-xs btn-success">
             <i class="mdi mdi-plus-thick" aria-hidden="true"></i>
         </a>
     {% endif %}
 {% endwith %}
 """
 
-VLAN_MEMBER_TAGGED = """
-{% if record.untagged_vlan_id == object.pk %}
-    <span class="text-danger"><i class="mdi mdi-close-thick"></i></span>
-{% else %}
-    <span class="text-success"><i class="mdi mdi-check-bold"></i></span>
-{% endif %}
-"""
-
-TENANT_LINK = """
-{% if record.tenant %}
-    <a href="{% url 'tenancy:tenant' slug=record.tenant.slug %}" title="{{ record.tenant.description }}">{{ record.tenant }}</a>
-{% elif record.vrf.tenant %}
-    <a href="{% url 'tenancy:tenant' slug=record.vrf.tenant.slug %}" title="{{ record.vrf.tenant.description }}">{{ record.vrf.tenant }}</a>*
-{% elif object.tenant %}
-    <a href="{% url 'tenancy:tenant' slug=object.tenant.slug %}" title="{{ object.tenant.description }}">{{ object.tenant }}</a>
-{% else %}
-    &mdash;
-{% endif %}
-"""
-
-
 #
 # VRFs
 #
@@ -149,7 +129,7 @@ class VRFTable(BaseTable):
     pk = ToggleColumn()
     name = tables.LinkColumn()
     rd = tables.Column(verbose_name="RD")
-    tenant = tables.TemplateColumn(template_code=COL_TENANT)
+    tenant = TenantColumn()
     enforce_unique = BooleanColumn(verbose_name="Unique")
     import_targets = tables.TemplateColumn(template_code=VRF_TARGETS, orderable=False)
     export_targets = tables.TemplateColumn(template_code=VRF_TARGETS, orderable=False)
@@ -179,7 +159,7 @@ class VRFTable(BaseTable):
 class RouteTargetTable(BaseTable):
     pk = ToggleColumn()
     name = tables.LinkColumn()
-    tenant = tables.TemplateColumn(template_code=COL_TENANT)
+    tenant = TenantColumn()
     tags = TagColumn(url_name="ipam:vrf_list")
 
     class Meta(BaseTable.Meta):
@@ -233,7 +213,7 @@ class RIRTable(BaseTable):
 class AggregateTable(BaseTable):
     pk = ToggleColumn()
     prefix = tables.LinkColumn(verbose_name="Aggregate", order_by=("network", "prefix_length"))
-    tenant = tables.TemplateColumn(template_code=TENANT_LINK)
+    tenant = TenantColumn()
     date_added = tables.DateColumn(format="Y-m-d", verbose_name="Added")
 
     class Meta(BaseTable.Meta):
@@ -319,8 +299,9 @@ class PrefixTable(StatusTableMixin, BaseTable):
         template_code=PREFIX_LINK, attrs={"td": {"class": "text-nowrap"}}, order_by=("network", "prefix_length")
     )
     vrf = tables.TemplateColumn(template_code=VRF_LINK, verbose_name="VRF")
-    tenant = tables.TemplateColumn(template_code=TENANT_LINK)
+    tenant = TenantColumn()
     site = tables.Column(linkify=True)
+    location = tables.Column(linkify=True)
     vlan = tables.Column(linkify=True, verbose_name="VLAN")
     role = tables.TemplateColumn(template_code=PREFIX_ROLE_LINK)
     is_pool = BooleanColumn(verbose_name="Pool")
@@ -335,6 +316,7 @@ class PrefixTable(StatusTableMixin, BaseTable):
             "vrf",
             "tenant",
             "site",
+            "location",
             "vlan",
             "role",
             "is_pool",
@@ -347,6 +329,7 @@ class PrefixTable(StatusTableMixin, BaseTable):
             "vrf",
             "tenant",
             "site",
+            "location",
             "vlan",
             "role",
             "description",
@@ -358,7 +341,7 @@ class PrefixTable(StatusTableMixin, BaseTable):
 
 class PrefixDetailTable(PrefixTable):
     utilization = tables.TemplateColumn(template_code=UTILIZATION_GRAPH, orderable=False)
-    tenant = tables.TemplateColumn(template_code=COL_TENANT)
+    tenant = TenantColumn()
     tags = TagColumn(url_name="ipam:prefix_list")
 
     class Meta(PrefixTable.Meta):
@@ -371,6 +354,7 @@ class PrefixDetailTable(PrefixTable):
             "utilization",
             "tenant",
             "site",
+            "location",
             "vlan",
             "role",
             "is_pool",
@@ -386,6 +370,7 @@ class PrefixDetailTable(PrefixTable):
             "utilization",
             "tenant",
             "site",
+            "location",
             "vlan",
             "role",
             "description",
@@ -404,7 +389,7 @@ class IPAddressTable(StatusTableMixin, BaseTable):
     )
     vrf = tables.TemplateColumn(template_code=VRF_LINK, verbose_name="VRF")
     role = ChoiceFieldColumn()
-    tenant = tables.TemplateColumn(template_code=TENANT_LINK)
+    tenant = TenantColumn()
     assigned_object = tables.Column(linkify=True, orderable=False, verbose_name="Interface")
     assigned_object_parent = tables.Column(
         accessor="assigned_object__parent",
@@ -434,7 +419,7 @@ class IPAddressTable(StatusTableMixin, BaseTable):
 
 class IPAddressDetailTable(IPAddressTable):
     nat_inside = tables.Column(linkify=True, orderable=False, verbose_name="NAT (Inside)")
-    tenant = tables.TemplateColumn(template_code=COL_TENANT)
+    tenant = TenantColumn()
     assigned = BooleanColumn(accessor="assigned_object_id", verbose_name="Assigned")
     tags = TagColumn(url_name="ipam:ipaddress_list")
 
@@ -491,7 +476,7 @@ class InterfaceIPAddressTable(StatusTableMixin, BaseTable):
 
     address = tables.LinkColumn(verbose_name="IP Address")
     vrf = tables.TemplateColumn(template_code=VRF_LINK, verbose_name="VRF")
-    tenant = tables.TemplateColumn(template_code=TENANT_LINK)
+    tenant = TenantColumn()
 
     class Meta(BaseTable.Meta):
         model = IPAddress
@@ -506,14 +491,15 @@ class InterfaceIPAddressTable(StatusTableMixin, BaseTable):
 class VLANGroupTable(BaseTable):
     pk = ToggleColumn()
     name = tables.Column(linkify=True)
-    site = tables.LinkColumn(viewname="dcim:site", args=[Accessor("site__slug")])
+    site = tables.Column(linkify=True)
+    location = tables.Column(linkify=True)
     vlan_count = LinkedCountColumn(viewname="ipam:vlan_list", url_params={"group": "slug"}, verbose_name="VLANs")
     actions = ButtonsColumn(model=VLANGroup, prepend_template=VLANGROUP_ADD_VLAN)
 
     class Meta(BaseTable.Meta):
         model = VLANGroup
-        fields = ("pk", "name", "site", "vlan_count", "slug", "description", "actions")
-        default_columns = ("pk", "name", "site", "vlan_count", "description", "actions")
+        fields = ("pk", "name", "site", "location", "vlan_count", "slug", "description", "actions")
+        default_columns = ("pk", "name", "site", "location", "vlan_count", "description", "actions")
 
 
 #
@@ -524,9 +510,10 @@ class VLANGroupTable(BaseTable):
 class VLANTable(StatusTableMixin, BaseTable):
     pk = ToggleColumn()
     vid = tables.TemplateColumn(template_code=VLAN_LINK, verbose_name="ID")
-    site = tables.LinkColumn(viewname="dcim:site", args=[Accessor("site__slug")])
-    group = tables.LinkColumn(viewname="ipam:vlangroup", args=[Accessor("group__pk")])
-    tenant = tables.TemplateColumn(template_code=COL_TENANT)
+    site = tables.Column(linkify=True)
+    location = tables.Column(linkify=True)
+    group = tables.Column(linkify=True)
+    tenant = TenantColumn()
     role = tables.TemplateColumn(template_code=VLAN_ROLE_LINK)
 
     class Meta(BaseTable.Meta):
@@ -535,6 +522,7 @@ class VLANTable(StatusTableMixin, BaseTable):
             "pk",
             "vid",
             "site",
+            "location",
             "group",
             "name",
             "tenant",
@@ -549,7 +537,7 @@ class VLANTable(StatusTableMixin, BaseTable):
 
 class VLANDetailTable(VLANTable):
     prefixes = tables.TemplateColumn(template_code=VLAN_PREFIXES, orderable=False, verbose_name="Prefixes")
-    tenant = tables.TemplateColumn(template_code=COL_TENANT)
+    tenant = TenantColumn()
     tags = TagColumn(url_name="ipam:vlan_list")
 
     class Meta(VLANTable.Meta):
@@ -557,6 +545,7 @@ class VLANDetailTable(VLANTable):
             "pk",
             "vid",
             "site",
+            "location",
             "group",
             "name",
             "prefixes",
@@ -570,6 +559,7 @@ class VLANDetailTable(VLANTable):
             "pk",
             "vid",
             "site",
+            "location",
             "group",
             "name",
             "prefixes",
@@ -586,7 +576,10 @@ class VLANMembersTable(BaseTable):
     """
 
     name = tables.LinkColumn(verbose_name="Interface")
-    tagged = tables.TemplateColumn(template_code=VLAN_MEMBER_TAGGED, orderable=False)
+    tagged = tables.Column(empty_values=(), orderable=False)
+
+    def render_tagged(self, value, record):
+        return render_boolean(record.untagged_vlan_id != self.context["object"].pk)
 
 
 class VLANDevicesTable(VLANMembersTable):
@@ -615,8 +608,9 @@ class InterfaceVLANTable(StatusTableMixin, BaseTable):
     vid = tables.LinkColumn(viewname="ipam:vlan", args=[Accessor("pk")], verbose_name="ID")
     tagged = BooleanColumn()
     site = tables.Column(linkify=True)
+    location = tables.Column(linkify=True)
     group = tables.Column(accessor=Accessor("group__name"), verbose_name="Group")
-    tenant = tables.TemplateColumn(template_code=COL_TENANT)
+    tenant = TenantColumn()
     role = tables.TemplateColumn(template_code=VLAN_ROLE_LINK)
 
     class Meta(BaseTable.Meta):
@@ -625,6 +619,7 @@ class InterfaceVLANTable(StatusTableMixin, BaseTable):
             "vid",
             "tagged",
             "site",
+            "location",
             "group",
             "name",
             "tenant",
