@@ -4,6 +4,7 @@ from django.contrib.auth.models import Group
 from django.contrib.contenttypes.models import ContentType
 from django.urls import reverse
 
+from nautobot.users.filters import GroupFilterSet
 from nautobot.users.models import ObjectPermission, Token
 from nautobot.utilities.testing import APIViewTestCases, APITestCase
 from nautobot.utilities.utils import deepmerge
@@ -54,6 +55,7 @@ class UserTest(APIViewTestCases.APIViewTestCase):
 
 class GroupTest(APIViewTestCases.APIViewTestCase):
     model = Group
+    filterset = GroupFilterSet
     view_namespace = "users"
     brief_fields = ["display", "id", "name", "url"]
     create_data = [
