@@ -1,24 +1,18 @@
-import json
 import uuid
 
 from django.conf import settings
-from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.models import ContentType
-from django.core.exceptions import FieldDoesNotExist, ObjectDoesNotExist
-from django.db.models import JSONField, ManyToManyField
-from django.forms.models import model_to_dict
-from django.test import Client, TestCase as _TestCase, override_settings, tag
+from django.core.exceptions import ObjectDoesNotExist
+from django.test import TestCase as _TestCase, override_settings, tag
 from django.urls import reverse, NoReverseMatch
 from django.utils.text import slugify
-from netaddr import IPNetwork
-from taggit.managers import TaggableManager
 
 from nautobot.extras.choices import CustomFieldTypeChoices, RelationshipSideChoices, ObjectChangeActionChoices
-from nautobot.extras.models import ChangeLoggedModel, ObjectChange, Tag
+from nautobot.extras.models import ChangeLoggedModel
 from nautobot.users.models import ObjectPermission
-from nautobot.utilities.permissions import resolve_permission_ct
-from nautobot.utilities.fields import JSONArrayField
-from .utils import disable_warnings, extract_form_failures, extract_page_body, post_data
+from nautobot.utilities.testing.mixins import NautobotTestCaseMixin
+from nautobot.utilities.utils import get_changes_for_model, get_filterset_for_model
+from .utils import disable_warnings, extract_page_body, post_data
 
 
 __all__ = (
@@ -29,173 +23,13 @@ __all__ = (
 )
 
 
-# Use the proper swappable User model
-User = get_user_model()
-
-
 @tag("unit")
-class TestCase(_TestCase):
+class TestCase(_TestCase, NautobotTestCaseMixin):
     """Base class for all Nautobot-specific unit tests."""
 
-    user_permissions = ()
-
     def setUp(self):
-
-        # Create the test user and assign permissions
-        self.user = User.objects.create_user(username="testuser")
-        self.add_permissions(*self.user_permissions)
-
-        # Initialize the test client
-        self.client = Client()
-
-        # Force login explicitly with the first-available backend
-        self.client.force_login(self.user)
-
-    def prepare_instance(self, instance):
-        """
-        Test cases can override this method to perform any necessary manipulation of an instance prior to its evaluation
-        against test data. For example, it can be used to decrypt a Secret's plaintext attribute.
-        """
-        return instance
-
-    def model_to_dict(self, instance, fields, api=False):
-        """
-        Return a dictionary representation of an instance.
-        """
-        # Prepare the instance and call Django's model_to_dict() to extract all fields
-        model_dict = model_to_dict(self.prepare_instance(instance), fields=fields)
-
-        # Map any additional (non-field) instance attributes that were specified
-        for attr in fields:
-            if hasattr(instance, attr) and attr not in model_dict:
-                model_dict[attr] = getattr(instance, attr)
-
-        for key, value in list(model_dict.items()):
-            try:
-                field = instance._meta.get_field(key)
-            except FieldDoesNotExist:
-                # Attribute is not a model field, but may be a computed field,
-                # so allow `field` checks to pass through.
-                field = None
-
-            # Handle ManyToManyFields
-            if value and isinstance(field, (ManyToManyField, TaggableManager)):
-
-                # Only convert ContentType to <app_label>.<model> for API serializers/views
-                if api and field.related_model is ContentType:
-                    model_dict[key] = sorted([f"{ct.app_label}.{ct.model}" for ct in value])
-                # Otherwise always convert object instances to pk
-                else:
-                    model_dict[key] = sorted([obj.pk for obj in value])
-
-            if api:
-
-                # Replace ContentType primary keys with <app_label>.<model>
-                if isinstance(getattr(instance, key), ContentType):
-                    ct = ContentType.objects.get(pk=value)
-                    model_dict[key] = f"{ct.app_label}.{ct.model}"
-
-                # Convert IPNetwork instances to strings
-                elif isinstance(value, IPNetwork):
-                    model_dict[key] = str(value)
-
-            else:
-
-                # Convert ArrayFields to CSV strings
-                if isinstance(field, JSONArrayField):
-                    model_dict[key] = ",".join([str(v) for v in value])
-
-                # Convert JSONField dict values to JSON strings
-                if isinstance(field, JSONField) and isinstance(value, dict):
-                    model_dict[key] = json.dumps(value)
-
-        return model_dict
-
-    #
-    # Permissions management
-    #
-
-    def add_permissions(self, *names):
-        """
-        Assign a set of permissions to the test user. Accepts permission names in the form <app>.<action>_<model>.
-        """
-        for name in names:
-            ct, action = resolve_permission_ct(name)
-            obj_perm = ObjectPermission(name=name, actions=[action])
-            obj_perm.save()
-            obj_perm.users.add(self.user)
-            obj_perm.object_types.add(ct)
-
-    #
-    # Custom assertions
-    #
-
-    def assertHttpStatus(self, response, expected_status, msg=None):
-        """
-        TestCase method. Provide more detail in the event of an unexpected HTTP response.
-        """
-        err_message = None
-        # Construct an error message only if we know the test is going to fail
-        if response.status_code != expected_status:
-            if hasattr(response, "data"):
-                # REST API response; pass the response data through directly
-                err = response.data
-            else:
-                # Attempt to extract form validation errors from the response HTML
-                form_errors = extract_form_failures(response.content.decode(response.charset))
-                err = form_errors or response.content.decode(response.charset) or "No data"
-            err_message = f"Expected HTTP status {expected_status}; received {response.status_code}: {err}"
-            if msg:
-                err_message = f"{msg}\n{err_message}"
-        self.assertEqual(response.status_code, expected_status, err_message)
-
-    def assertInstanceEqual(self, instance, data, exclude=None, api=False):
-        """
-        Compare a model instance to a dictionary, checking that its attribute values match those specified
-        in the dictionary.
-
-        :param instance: Python object instance
-        :param data: Dictionary of test data used to define the instance
-        :param exclude: List of fields to exclude from comparison (e.g. passwords, which get hashed)
-        :param api: Set to True is the data is a JSON representation of the instance
-        """
-        if exclude is None:
-            exclude = []
-
-        fields = [k for k in data.keys() if k not in exclude]
-        model_dict = self.model_to_dict(instance, fields=fields, api=api)
-
-        new_model_dict = {}
-        for k, v in model_dict.items():
-            if isinstance(v, list):
-                # Sort lists of values. This includes items like tags, or other M2M fields
-                new_model_dict[k] = sorted(v)
-            else:
-                new_model_dict[k] = v
-
-        # Omit any dictionary keys which are not instance attributes or have been excluded
-        relevant_data = {}
-        for k, v in data.items():
-            if hasattr(instance, k) and k not in exclude:
-                if isinstance(v, list):
-                    # Sort lists of values. This includes items like tags, or other M2M fields
-                    relevant_data[k] = sorted(v)
-                else:
-                    relevant_data[k] = v
-
-        self.assertEqual(new_model_dict, relevant_data)
-
-    #
-    # Convenience methods
-    #
-
-    @classmethod
-    def create_tags(cls, *names):
-        """
-        Create and return a Tag instance for each name given.
-        """
-
-        return [Tag.objects.create(name=name, slug=slugify(name)) for name in names]
+        """Initialize user and client."""
+        super().setUpNautobot()
 
 
 class ModelTestCase(TestCase):
@@ -300,7 +134,7 @@ class ViewTestCases:
 
             # Try GET without permission
             with disable_warnings("django.request"):
-                self.assertHttpStatus(self.client.get(instance.get_absolute_url()), 403)
+                self.assertHttpStatus(self.client.get(instance.get_absolute_url()), [403, 404])
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
         def test_get_object_with_permission(self):
@@ -342,6 +176,7 @@ class ViewTestCases:
             if self.custom_fields:
                 for custom_field in self.custom_fields:
                     self.assertIn(str(custom_field), response_body, msg=response_body)
+                    # 2.0 TODO: #824 custom_field.slug rather than custom_field.name
                     if custom_field.type == CustomFieldTypeChoices.TYPE_MULTISELECT:
                         for value in instance.cf.get(custom_field.name):
                             self.assertIn(str(value), response_body, msg=response_body)
@@ -398,6 +233,18 @@ class ViewTestCases:
             response = self.client.get(url)
             self.assertHttpStatus(response, 200)
 
+    class GetObjectNotesViewTestCase(ModelViewTestCase):
+        """
+        View the notes for an instance.
+        """
+
+        @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+        def test_get_object_notes(self):
+            if hasattr(self.model, "notes"):
+                url = self._get_url("notes", self._get_queryset().first())
+                response = self.client.get(url)
+                self.assertHttpStatus(response, 200)
+
     class CreateObjectViewTestCase(ModelViewTestCase):
         """
         Create a single new instance.
@@ -407,6 +254,7 @@ class ViewTestCases:
 
         form_data = {}
         slug_source = None
+        slugify_function = staticmethod(slugify)
         slug_test_object = ""
 
         def test_create_object_without_permission(self):
@@ -453,10 +301,7 @@ class ViewTestCases:
 
             if hasattr(self.model, "to_objectchange"):
                 # Verify ObjectChange creation
-                objectchanges = ObjectChange.objects.filter(
-                    changed_object_type=ContentType.objects.get_for_model(instance),
-                    changed_object_id=instance.pk,
-                )
+                objectchanges = get_changes_for_model(instance)
                 self.assertEqual(len(objectchanges), 1)
                 self.assertEqual(objectchanges[0].action, ObjectChangeActionChoices.ACTION_CREATE)
 
@@ -506,7 +351,7 @@ class ViewTestCases:
             # This really should go on a models test page, but we don't have test structures for models.
             if self.slug_source is not None:
                 object = self.model.objects.get(**{self.slug_source: self.slug_test_object})
-                expected_slug = slugify(getattr(object, self.slug_source))
+                expected_slug = self.slugify_function(getattr(object, self.slug_source))
                 self.assertEqual(object.slug, expected_slug)
 
         def test_slug_not_modified(self):
@@ -514,7 +359,7 @@ class ViewTestCases:
             # This really should go on a models test page, but we don't have test structures for models.
             if self.slug_source is not None:
                 object = self.model.objects.get(**{self.slug_source: self.slug_test_object})
-                expected_slug = slugify(getattr(object, self.slug_source))
+                expected_slug = self.slugify_function(getattr(object, self.slug_source))
                 # Update slug source field str
                 filter = self.slug_source + "__contains"
                 self.model.objects.filter(**{filter: self.slug_test_object}).update(**{self.slug_source: "Test"})
@@ -537,7 +382,7 @@ class ViewTestCases:
 
             # Try GET without permission
             with disable_warnings("django.request"):
-                self.assertHttpStatus(self.client.get(self._get_url("edit", instance)), 403)
+                self.assertHttpStatus(self.client.get(self._get_url("edit", instance)), [403, 404])
 
             # Try POST without permission
             request = {
@@ -545,7 +390,7 @@ class ViewTestCases:
                 "data": post_data(self.form_data),
             }
             with disable_warnings("django.request"):
-                self.assertHttpStatus(self.client.post(**request), 403)
+                self.assertHttpStatus(self.client.post(**request), [403, 404])
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
         def test_edit_object_with_permission(self):
@@ -570,10 +415,7 @@ class ViewTestCases:
 
             if hasattr(self.model, "to_objectchange"):
                 # Verify ObjectChange creation
-                objectchanges = ObjectChange.objects.filter(
-                    changed_object_type=ContentType.objects.get_for_model(instance),
-                    changed_object_id=instance.pk,
-                )
+                objectchanges = get_changes_for_model(instance)
                 self.assertEqual(len(objectchanges), 1)
                 self.assertEqual(objectchanges[0].action, ObjectChangeActionChoices.ACTION_UPDATE)
 
@@ -622,7 +464,7 @@ class ViewTestCases:
 
             # Try GET without permission
             with disable_warnings("django.request"):
-                self.assertHttpStatus(self.client.get(self._get_url("delete", instance)), 403)
+                self.assertHttpStatus(self.client.get(self._get_url("delete", instance)), [403, 404])
 
             # Try POST without permission
             request = {
@@ -630,7 +472,7 @@ class ViewTestCases:
                 "data": post_data({"confirm": True}),
             }
             with disable_warnings("django.request"):
-                self.assertHttpStatus(self.client.post(**request), 403)
+                self.assertHttpStatus(self.client.post(**request), [403, 404])
 
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
         def test_delete_object_with_permission(self):
@@ -656,10 +498,7 @@ class ViewTestCases:
 
             if hasattr(self.model, "to_objectchange"):
                 # Verify ObjectChange creation
-                objectchanges = ObjectChange.objects.filter(
-                    changed_object_type=ContentType.objects.get_for_model(instance),
-                    changed_object_id=instance.pk,
-                )
+                objectchanges = get_changes_for_model(instance)
                 self.assertEqual(len(objectchanges), 1)
                 self.assertEqual(objectchanges[0].action, ObjectChangeActionChoices.ACTION_DELETE)
 
@@ -705,6 +544,11 @@ class ViewTestCases:
         Retrieve multiple instances.
         """
 
+        filterset = None
+
+        def get_filterset(self):
+            return self.filterset or get_filterset_for_model(self.model)
+
         @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
         def test_list_objects_anonymous(self):
             # Make the request as an unauthenticated user
@@ -712,6 +556,66 @@ class ViewTestCases:
             response = self.client.get(self._get_url("list"))
             self.assertHttpStatus(response, 200)
 
+        @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"])
+        def test_list_objects_filtered(self):
+            instance1, instance2 = self._get_queryset().all()[:2]
+            response = self.client.get(f"{self._get_url('list')}?id={instance1.pk}")
+            self.assertHttpStatus(response, 200)
+            content = extract_page_body(response.content.decode(response.charset))
+            # TODO: it'd make test failures more readable if we strip the page headers/footers from the content
+            if hasattr(self.model, "name"):
+                self.assertIn(instance1.name, content, msg=content)
+                self.assertNotIn(instance2.name, content, msg=content)
+            try:
+                self.assertIn(self._get_url("view", instance=instance1), content, msg=content)
+                self.assertNotIn(self._get_url("view", instance=instance2), content, msg=content)
+            except NoReverseMatch:
+                pass
+
+        @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"], STRICT_FILTERING=True)
+        def test_list_objects_unknown_filter_strict_filtering(self):
+            """Verify that with STRICT_FILTERING, an unknown filter results in an error message and no matches."""
+            instance1, instance2 = self._get_queryset().all()[:2]
+            response = self.client.get(f"{self._get_url('list')}?ice_cream_flavor=chocolate")
+            self.assertHttpStatus(response, 200)
+            content = extract_page_body(response.content.decode(response.charset))
+            # TODO: it'd make test failures more readable if we strip the page headers/footers from the content
+            self.assertIn("Unknown filter field", content, msg=content)
+            if hasattr(self.model, "name"):
+                self.assertNotIn(instance1.name, content, msg=content)
+                self.assertNotIn(instance2.name, content, msg=content)
+            try:
+                self.assertNotIn(self._get_url("view", instance=instance1), content, msg=content)
+                self.assertNotIn(self._get_url("view", instance=instance2), content, msg=content)
+            except NoReverseMatch:
+                pass
+
+        @override_settings(EXEMPT_VIEW_PERMISSIONS=["*"], STRICT_FILTERING=False)
+        def test_list_objects_unknown_filter_no_strict_filtering(self):
+            """Verify that without STRICT_FILTERING, an unknown filter is ignored."""
+            instance1, instance2 = self._get_queryset().all()[:2]
+            with self.assertLogs("nautobot.utilities.filters") as cm:
+                response = self.client.get(f"{self._get_url('list')}?ice_cream_flavor=chocolate")
+            self.assertEqual(
+                cm.output,
+                [
+                    f"WARNING:nautobot.utilities.filters:{self.get_filterset().__name__}: "
+                    'Unknown filter field "ice_cream_flavor"',
+                ],
+            )
+            self.assertHttpStatus(response, 200)
+            content = extract_page_body(response.content.decode(response.charset))
+            # TODO: it'd make test failures more readable if we strip the page headers/footers from the content
+            self.assertNotIn("Unknown filter field", content, msg=content)
+            if hasattr(self.model, "name"):
+                self.assertIn(instance1.name, content, msg=content)
+                self.assertIn(instance2.name, content, msg=content)
+            try:
+                self.assertIn(self._get_url("view", instance=instance1), content, msg=content)
+                self.assertIn(self._get_url("view", instance=instance2), content, msg=content)
+            except NoReverseMatch:
+                pass
+
         @override_settings(EXEMPT_VIEW_PERMISSIONS=[])
         def test_list_objects_without_permission(self):
 
@@ -946,10 +850,6 @@ class ViewTestCases:
                 "_apply": True,  # Form button
             }
 
-            # Test GET without permission
-            with disable_warnings("django.request"):
-                self.assertHttpStatus(self.client.get(self._get_url("bulk_edit")), 403)
-
             # Try POST without permission
             with disable_warnings("django.request"):
                 self.assertHttpStatus(self.client.post(self._get_url("bulk_edit"), data), 403)
@@ -1029,10 +929,6 @@ class ViewTestCases:
                 "_confirm": True,  # Form button
             }
 
-            # Test GET without permission
-            with disable_warnings("django.request"):
-                self.assertHttpStatus(self.client.get(self._get_url("bulk_delete")), 403)
-
             # Try POST without permission
             with disable_warnings("django.request"):
                 self.assertHttpStatus(self.client.post(self._get_url("bulk_delete"), data), 403)
@@ -1172,6 +1068,7 @@ class ViewTestCases:
     class PrimaryObjectViewTestCase(
         GetObjectViewTestCase,
         GetObjectChangelogViewTestCase,
+        GetObjectNotesViewTestCase,
         CreateObjectViewTestCase,
         EditObjectViewTestCase,
         DeleteObjectViewTestCase,
@@ -1189,6 +1086,7 @@ class ViewTestCases:
     class OrganizationalObjectViewTestCase(
         GetObjectViewTestCase,
         GetObjectChangelogViewTestCase,
+        GetObjectNotesViewTestCase,
         CreateObjectViewTestCase,
         EditObjectViewTestCase,
         DeleteObjectViewTestCase,
@@ -1219,6 +1117,7 @@ class ViewTestCases:
     class DeviceComponentViewTestCase(
         GetObjectViewTestCase,
         GetObjectChangelogViewTestCase,
+        GetObjectNotesViewTestCase,
         EditObjectViewTestCase,
         DeleteObjectViewTestCase,
         ListObjectsViewTestCase,
